#ifndef __DIC32_H__
#define __DIC32_H__
// ====================================================================

// 1. Search for dongle device(s) connected to the computer
EXTERN_C int WINAPI DIC_Find();
// Return value:
// it is < 0, indicating an error code is returned, for details, please see below.
// If it is = 0, no card reader device exists
// If it is >0, the return value is the number of card reader device(s) 

//(Note: Normally, even if the user does not connect any device, the driver program will report the card presence 
//due to the existence of the driver. It is necessary to confirm the real device presence with command DIC_Open.) 
//Suppose that maximum 32 card reader devices can be connected to a computer. 

// ====================================================================

// 2. Open the specified card reader
EXTERN_C int WINAPI DIC_Open(int hic, char* reader_name);
// Input:
// hic	Card handle,it's a integer value.
// (1)positive value, it is a enumeration valueany one of 0, 1, 2, ... return value of DIC_Find(). If reader_name 
//     is not null, card reader name is returned.
// (2)negative valuereader_name parameter can not be nullDIC_Open will attempt to open the card reader directly 
//     by the device name in reader_name; in case of a virtual device, it is the path directory of the file to be opened. 

// reader_name  name of card reader.
// An input/output parameter, see comments on hic

// Return value:
// >=0				Success, the opened handle is returned
// <0				An error code is returned, for detailed descriptions, please refer to code defined below.
// ====================================================================

// 3. Close the specified card reader
EXTERN_C int WINAPI DIC_Close(int hic);
// Input:
// hic: Card handle, must be consistent with the hic returned by DIC_Open. 
// Return:
// An error code , for detailed descriptions, please refer to code defined below.

// ====================================================================

// 4. Create virtual IC card file(*virtual card private instruction)
EXTERN_C int WINAPI DIC_Create(char* filename, char* mfname, char* atr, WORD atrlen);
// Input:
// filename:	Path file name to be created 
// mfname:		Main volume name
// atr:			Software developer information 
// atrlen:		Software developer information length

// Return :
// An error code , for detailed descriptions, please refer to code defined below.

// ======================================================================

// 5. Get the dynamic library version
EXTERN_C int WINAPI DIC_GetVersion(char* ver);
// Input:
// ver: returns the version number of the dynamic library, the version content is:
//    [V/R]xx.yy
// For example, R01.10 means it is "Real card dynamic library Version 1.10 "
//              R03.02 indicats "Virtual card dynamic library Version 3.02 "

// Return:
//	An error code , for detailed descriptions, please refer to code defined below.

// ======================================================================

// 6. Dongle operation command functions
EXTERN_C int WINAPI DIC_Command(int hic, int cmd, void* cmddata);
// Input:
// hic		device handle
// cmd		specific command
// cmddata	data relating to command
EXTERN_C int WINAPI DIC_FindByMgrCode(void * pMgrCode);

EXTERN_C int WINAPI EnterMultThreadMode();
// Command definition
#define GET_CARD_INFO				0x02000000
#define GET_HARDWARE_INFO			0x00FF1C00
#define GET_MANAGER_CODE			0x00FF1A00
#define GET_CARD_PRIVILEGE			0x00FF2200
#define GET_REMOTE_INFO				0x00FF1e00
#define SET_REMOTE_INFO				0x00FF1003
#define CHECK_REMOTE_INFO			0x01FF1003
#define GET_CURRENT_DIR				0x00001800
#define SET_CURRENT_DIR				0x0000a400
#define GET_CURRENT_FILE			0x00001801
#define SET_CURRENT_FILE			0x0100a400
#define GET_PARENT_DIR				0x00001802
#define SET_PARENT_DIR				0x0000a402
#define LIST_DIR					0x0000b200
#define	READ_FILE					0x0000b000
#define WRITE_FILE					0x0000d000
#define FORMAT_CARD					0x0000e000
#define CREATE_DIR					0x0000e001
#define CREATE_FILE					0x0100e001
#define REMOVE_DIR					0x0000ee00
#define REMOVE_FILE					0x0000ee01
#define RANDOM						0x00008400
#define RUN							0x03000000
#define CRYPTOTEXT_FILE				0x04000000
#define PLAINTEXT_FILE				0x05000000

#define DESENC						0x00005000
#define DESDEC						0x00005100
#define RSAGENKEY					0x00003300
#define RSAENC						0x00003400
#define RSADEC						0x00003500

#define GETFREESPACE				0X00003000
#define MODULE_GETFILEFLAG			0X00003600
#define MODULE_CRYPTOTEXT_MODFLAG	0x00003700
#define MODULE_PLAINTEXT_MODFLAG	0x00003800
#define MODULE_ENCRYPT_CONTENT		0x00003900

#define SETCOUNTER				0x00005700
#define STEPCOUNTER					0x00005800

#define CHECK_SUPER_PASS			0x00FF1002
#define SET_SUPER_PASS				0x00FFF402
#define GET_UPGRADE_REMOTE_PASS		0x00FF2000

// "data": parameters relating to command

// Data structure of "data" in GET_CARD_INFO command
typedef struct
{
	char volume[16];
	char atr[15];
} DICST_CardInfo;

// Data structure of "data" in GET_HARDWARE_INFO command
typedef struct
{
	DWORD FactoryTime;
	DWORD HardSerial;
	DWORD ShipTime;
	DWORD COSVersion;
} DICST_HardInfo;

// Data structure of "data" in GET_MANAGER_CODE command
typedef struct
{
	WORD Zone;
	WORD Agent;
	WORD User1;
	WORD User2;
} DICST_ManagerCode;

// There is only 1 byte in "data" in GET_CARD_PRIVILEGE command
// the current password verification status is represented by bits
#define DICPR_NOPASS		0
#define	DICPR_SUPERPASS		4
#define	DICPR_REMOTEPASS	8

// Data structure of "data" in GET_REMOTE_INFO, SET_REMOTE_INFO, CHECK_REMOTE_INFO command
typedef struct
{
	DWORD RemoteTag;
	BYTE RemotePass[8];
} DICST_RemoteInfo;

// Data structure of "data" in GET_UPGRADE_REMOTE_PASS command
typedef struct
{
	DWORD RemoteTag;
	DWORD HardSerial;
	BYTE RemotePass[8];
} DICST_Upgrade_RemotePass;

// Data structure of "data" in GET_CURRENT_DIR, SET_CURRENT_DIR, GET_PARENT_DIR, CREATE_DIR command
typedef struct
{
	WORD		dirid;			// directory ID
	BYTE		dircla;			// directory class
	BYTE		diratrpri;		// directory attributes & directory security level
	WORD		dirsize;		// unused
	char		dirname[16];	// directory name
} DICST_Dir;

// Data structure of "data" in GET_CURRENT_FILE, SET_CURRENT_FILE, CREATE_FILE command
typedef struct
{
	WORD		fileid;			// file ID
	BYTE		filecla;		// file class
	BYTE		fileatrpri;		// file attributes & file security level
	WORD		filesize;		// file size
	char		filename[17];	// file name
	char        filepath[128];
} DICST_File;

// file attribute definition
#define FILEATTR_NORMAL			0x00
#define FILEATTR_EXEC			0x10	// means executable
#define FILEATTR_DIR			0x20	// means directory file
#define FILEATTR_UPIGNORE		0x40	// means the program will not be executed if current security level is higher than the level of the executable itself
#define FILEATTR_INTERNAL		0x80	// means a file used by internal applications; cannot be operated externally
                                        // if an executable has internal attributes configured, the supper password is required for displaying it when listing directory
// Note: The structure returned by LIST_DIR command can be DICST_Dir or DICST_File.
// The particular structure can be determined by FILEATTR_DIR.

// Data structure of "data" in READ_FILE command 
typedef struct
{
	WORD		offset;
	WORD		size;
} DICST_Before_Read_Data;

// Returned data structure of "data" in READ_FILE command
typedef struct
{
	WORD		readedsize;
	char		buffer[1];		// the size is readsize
} DICST_After_Read_Data;

// Data structure of "data" in WRITE_FILE command
typedef struct
{
	WORD		offset;
	WORD		size;
	char		buffer[1];		// the size is size
} DICST_Write_Data;

// Data structure of "data" in FORMAT_CARD command
typedef struct
{
	char		volume[16];
	char		atr[15];
	BOOL        isdes;
	BOOL        isfloat;
	BOOL        isExtend;
	BYTE		bModify;
} DICST_System_Info;

// the "data" structure required by RUN_DATA is:
typedef struct
{
	WORD RunID;
	WORD ParaSize;
	BYTE Para[1];	// the size is ParaSize
} DICST_Before_Run_Data;

// When RUN_DATA returns parameters, the "data" structure is:
typedef struct
{
	WORD ResultSize;
	BYTE Result[1]; // the size is ResultSize
} DICST_After_Run_Data;

// Data structure of "data" in SET_SUPER_PASS command
typedef struct
{
	BYTE MaxTryTimes;
	BYTE SuperPass[8];
} DICST_SuperPass_Data;

// Data structure of "data" in  CRYPTOTEXT_FILE command
typedef struct
{
	WORD		fileid;			// file ID
	BYTE		filecla;		// file class
	BYTE		fileatrpri;		// file attributes & file security level
	WORD		filesize;		// file size
	char		filename[16];	// file name
	BYTE		filedata[1];	// file content, the size is filesize
} DICST_Before_CryptoFile_Data;

// When CRYPTOTEXT_FILE returns parameters, the "data" structure is:
typedef struct
{
	WORD ResultSize;
	BYTE Result[1]; // the size is ResultSize
} DICST_After_CryptoFile_Data;

// 3 structs that must be filled by encryption and decryption algorithm commands
typedef struct 
{								//for encryption		//for decryption
	WORD dataLength;			//length of data encrypted	//length of data decrypted
	WORD KeyFileid;				//file ID of DES key file
	BYTE KeyIndex;				//identifier of the key used for this encryption in the DES key file
								//Note: A DES key file contains several keys used for DES encryption/decryption.
								//the key used for each decryption is identified by KeyIndex

	char data[1];				//encryption data			//decryption data
} DICST_Des_Data;
typedef struct 
{								//for encryption		//for decryption
	WORD	KeyFileid;			//file ID of public key file	//file ID of private key file
	WORD	dataLength;			//length of encryption data	//length of decryption data
	char	data[1];			//encryption data			//decryption data
} DICST_Rsa_Data;
typedef struct 
{
	WORD pubKeyID;				//file ID of public key file
	WORD pubKeySize;			//size of public key 512/1024
	WORD pivKeyID;				//file ID of private key file
} DICST_Rsa_GenKey;
// 3 data structures returned by encryption algorithm calls
typedef struct 
{								//for encryption			//for decryption
	WORD dataLength;			//length of returned data after encryption	//length of returned data after decryption
	char data[1];			//returned data after encryption		//returned data after decryption
} DICST_After_EncDec_Data;

// get module enabling flag structure
typedef struct 
{
	BYTE NumFileID;		//actual BYTE length of executable file ID sequence
	WORD fileID[1];		//executable file ID sequence to read
}
DICST_Module_GetFLag;
typedef BYTE* FileFlag;	//return value after getting module enabling flag


typedef struct  
{
	WORD fileID;	//file ID
	BYTE flag;		//file enabling flag
}
DICST_PlainText_OneMod;
typedef struct
{
	BYTE dataLen;
	DICST_PlainText_OneMod data[1];
}
DICST_PlainText_ModuleData;

typedef struct 
{
	BYTE numExe;		//number of executable files, the maximum is 
	BYTE flag;			//the enabling flag of the specified executable files as follows
	WORD exeFileID[3];	//executable file ID
}
DICST_CryptoText_OneMod;
typedef struct  
{
	BYTE nDataLen;
	DICST_CryptoText_OneMod plaintextModuleData[1];
}
DICST_CryptoText_Encrypt;

typedef struct
{
	BYTE dataLen;
	BYTE data[1];
}
DICST_CryptoText_ModuleData;

// ====================================================================

// 7. A function to help users extract data from the "data" of DIC_Command command
EXTERN_C int WINAPI DIC_Get(void* xdata, int p1, int p2, char* buffer);
// Input:
// xdata	target buffer
// p1		mode/offset; the highest bit of 1 means mode operation; otherwise, it is an offset value defined by the user
// p2		return method/size, the highest 2 bits mean the return method; the remaining bits mean the size of the operation
// buffer	the char type buffer for extracted data

// ====================================================================

// 8. A function to help users to specify data in the "data" in DIC_Command command
EXTERN_C int WINAPI DIC_Set(void* xdata, int p1, int p2, int p3, char* buffer);
// Input:
// xdata	target buffer for storage
// p1		mode/offset; the highest bit of 1 means mode operation; otherwise, it is an offset value defined by the user
// p2		passing method/size, the highest 2 bits mean the passing method; the remaining bits mean the size of the operation
// p3		generally, the value(s) the user wants to pass
// buffer	the char type buffer to store data

// ====================================================================
// For p1 of DIC_Get and DIC_Set, some mode operations are pre-defined.
// The highest bit of these mode operations is set to 1
#define FILL					0x80000000
#define ATR						0x80010000
#define VOLUME					0x80020000
#define FACTORY_TIME			0x80030000
#define HARD_SERIAL				0x80040000
#define SHIP_TIME				0x80050000
#define COS_VERSION				0x80060000
#define ZONE					0x80070000
#define AGENT					0x80080000
#define USER1					0x80090000
#define USER2					0x800A0000
#define DIR_ID					0x800B0000
#define DIR_CLASS				0x800C0000
#define DIR_ATTRIBUTE			0x800D0000
//#define DIR_PRIVILEGE			0x800E0000	directory has no security level
#define FILE_NO					0x800F0000
#define DIR_NAME				0x80100000
#define FILE_ID					0x80110000
#define FILE_CLASS				0x80120000
#define FILE_ATTRIBUTE			0x80130000
#define FILE_PRIVILEGE			0x80140000
#define FILE_SIZE				0x80150000
#define FILE_NAME				0x80160000
#define WRITE_DATA				0x80170000
#define READ_DATA				0x80180000
#define RUN_DATA				0x80190000
#define RANDOM_SIZE				0x801A0000
#define REMOTE_TAG				0x801B0000
#define REMOTE_PASS				0x801C0000
#define UPGRADE_HARD_SERIAL		0x801D0000
#define UPGRADE_REMOTE_TAG		0x801E0000
#define UPGRADE_REMOTE_PASS		0x801F0000
#define REMOTEPASS_STATUS		0x80200000
#define SUPERPASS_STATUS		0x80210000
#define SUPERPASS_DATA			0x80220000
#define FILE_DATA				0x80230000
#define RELOAD_FLOATLIBS            0x0000e002
#define RELOAD_RSADESLIBS           0x0000e003
#define ISLOADEDFLOATLIBS           0x0000e004
#define ISLOADEDRSADESLIBS          0x0000e005 
#define RELOAD_EXTENDFLOATLIBS      0x0000e006
#define ISLOADEXTENDFLOATLIBS       0x0000e007
#define WRITE_HARDSERIAL            0x0000e008

//DICST_Des_Data			//Used by DIC_Set
#define DES_KEYFILEID			0x80240000
#define DES_KEYINDEX			0x80250000
#define DES_DATA				0x80260000

//DICST_After_EncDec_Data	//Used by DIC_Get
#define AFTER_ENCDEC_DATA		0x80270000

//DICST_Rsa_Data			//Used by DIC_Set
#define RSA_KEYFILEID			0x80280000
#define RSA_DATA				0x80290000

//DICST_Rsa_GenKey			//Used by Dic_Set
#define RSA_PUBKEYID			0x802A0000
#define RSA_PUBKEYSIZE			0x802B0000
#define RSA_PIVKEYID			0x802C0000

// ====================================================================
// For p2 of DIC_Get and DIC_Set, the highest 2 bits mean the passing or returning method of data
#define BY_VALUE				0x00000000
#define BY_DOUBLE				0x40000000
#define BY_ARRAY				0x80000000
#define BY_STRING				0xC0000000

// For DIC_GetData:
// BY_VALUE		means the data is returned via a return value
// BY_DOUBLE	means if the returned data is a floating number, it will be converted to a string and placed into a buffer to return
// BY_ARRAY		means the returned data is placed into a buffer to return
// BY_STRING	means if the returned data is a number, it will be converted to a string and placed into a buffer to return

// For DIC_SetData:
// BY_VALUE		means the data is input via p3
// BY_DOUBLE	means the data is input via buffer, and the data in buffer is a string representation of floating number
// BY_ARRAY		means the data is input via buffer
// BY_STRING	means the data is input via buffer, and the data in buffer is in string representation
// ====================================================================


// Error code ===========================================================

#ifndef SCARD_S_SUCCESS

// Success
#define SCARD_S_SUCCESS 			((DWORD)0x00000000L)

// Internal link check failure
#define SCARD_F_INTERNAL_ERROR			((DWORD)0x80100001L)

// Operation is terminated by user
#define SCARD_E_CANCELLED			((DWORD)0x80100002L)

// Incorrect operation handle
#define SCARD_E_INVALID_HANDLE			((DWORD)0x80100003L)

// Incorrect parameter
#define SCARD_E_INVALID_PARAMETER		((DWORD)0x80100004L)

// Registered startup information lost or invalid 
#define SCARD_E_INVALID_TARGET			((DWORD)0x80100005L)

// Memory not enough for command completion 
#define SCARD_E_NO_MEMORY			((DWORD)0x80100006L)

// Internal overtime
#define SCARD_F_WAITED_TOO_LONG			((DWORD)0x80100007L)

// User specified buffer is too small, not enough for the return data
#define SCARD_E_INSUFFICIENT_BUFFER		((DWORD)0x80100008L)

// Unknown card reader
#define SCARD_E_UNKNOWN_READER			((DWORD)0x80100009L)

// Over user specified time
#define SCARD_E_TIMEOUT				((DWORD)0x8010000AL)

// Card occupied by other connection
#define SCARD_E_SHARING_VIOLATION		((DWORD)0x8010000BL)

// No card in the card reader
#define SCARD_E_NO_SMARTCARD			((DWORD)0x8010000CL)

// Unknown card type
#define SCARD_E_UNKNOWN_CARD			((DWORD)0x8010000DL)

// Card reader can not complete card eject operation
#define SCARD_E_CANT_DISPOSE			((DWORD)0x8010000EL)

// Current card does not support user specified communication protocol
#define SCARD_E_PROTO_MISMATCH			((DWORD)0x8010000FL)

// Card is not ready for receiving commands
#define SCARD_E_NOT_READY			((DWORD)0x80100010L)

// Some variable values are not suitable
#define SCARD_E_INVALID_VALUE			((DWORD)0x80100011L)

// Operation is terminated by system, it may be necessary to log on again or shut down. 
#define SCARD_E_SYSTEM_CANCELLED		((DWORD)0x80100012L)

// Internal communication error
#define SCARD_F_COMM_ERROR			((DWORD)0x80100013L)

// Unknown internal error
#define SCARD_F_UNKNOWN_ERROR			((DWORD)0x80100014L)

// Invalid vendor information
#define SCARD_E_INVALID_ATR			((DWORD)0x80100015L)

// User attempts to terminate some process not available
#define SCARD_E_NOT_TRANSACTED			((DWORD)0x80100016L)

// The specifiied card reader is not ready
#define SCARD_E_READER_UNAVAILABLE		((DWORD)0x80100017L)

// Operation is terminated, service program is allowed to exit
#define SCARD_P_SHUTDOWN			((DWORD)0x80100018L)

// PCI receiving buffer is too small
#define SCARD_E_PCI_TOO_SMALL			((DWORD)0x80100019L)

// Card reader drive does not support the current card reader
#define SCARD_E_READER_UNSUPPORTED		((DWORD)0x8010001AL)

// Card reader driver program can not create an exclusive name, card reader with the same name already exists
#define SCARD_E_DUPLICATE_READER		((DWORD)0x8010001BL)

// Card is not supported by the current card reader
#define SCARD_E_CARD_UNSUPPORTED		((DWORD)0x8010001CL)

// Smart card service is not enabled
#define SCARD_E_NO_SERVICE			((DWORD)0x8010001DL)

// Smart card service is disabled
#define SCARD_E_SERVICE_STOPPED			((DWORD)0x8010001EL)

// Some unexpected smart card error
#define SCARD_E_UNEXPECTED			((DWORD)0x8010001FL)

// Smart card software developer information not accessible
#define SCARD_E_ICC_INSTALLATION		((DWORD)0x80100020L)

// Smart card vendor information not accessible
#define SCARD_E_ICC_CREATEORDER			((DWORD)0x80100021L)

// Current smart card does not support the function user demanded
#define SCARD_E_UNSUPPORTED_FEATURE		((DWORD)0x80100022L)

// The specified directory does not exist
#define SCARD_E_DIR_NOT_FOUND			((DWORD)0x80100023L)

// The specified file does not exist
#define SCARD_E_FILE_NOT_FOUND			((DWORD)0x80100024L)

// The specified directory is no longer a valid directory
#define SCARD_E_NO_DIR				((DWORD)0x80100025L)

// The specified file is no longer a valid file, no file selected
#define SCARD_E_NO_FILE				((DWORD)0x80100026L)

// The file denies any access
#define SCARD_E_NO_ACCESS			((DWORD)0x80100027L)

// Card is full, no more data write
#define SCARD_E_WRITE_TOO_MANY			((DWORD)0x80100028L)

// File pointer error
#define SCARD_E_BAD_SEEK			((DWORD)0x80100029L)

// PIN code error
#define SCARD_E_INVALID_CHV			((DWORD)0x8010002AL)

// An unrecognized error code is returned from smart card service
#define SCARD_E_UNKNOWN_RES_MNG			((DWORD)0x8010002BL)

// The certificate requested does not exist
#define SCARD_E_NO_SUCH_CERTIFICATE		((DWORD)0x8010002CL)

// The certificate requested is not available
#define SCARD_E_CERTIFICATE_UNAVAILABLE		((DWORD)0x8010002DL)

// No card reader found
#define SCARD_E_NO_READERS_AVAILABLE		((DWORD)0x8010002EL)

// Data loss during smart card communication, please try again
#define SCARD_E_COMM_DATA_LOST			((DWORD)0x8010002FL)

// The key file requested does not exist
#define SCARD_E_NO_KEY_CONTAINER		((DWORD)0x80100030L)

// Card reader can not communicate with the card due to software developer information conflict
#define SCARD_W_UNSUPPORTED_CARD		((DWORD)0x80100065L)

// Card does not respond to reset
#define SCARD_W_UNRESPONSIVE_CARD		((DWORD)0x80100066L)

// Card no power
#define SCARD_W_UNPOWERED_CARD			((DWORD)0x80100067L)

// Card is reset, shared information is invalid
#define SCARD_W_RESET_CARD			((DWORD)0x80100068L)

// Card is removed
#define SCARD_W_REMOVED_CARD			((DWORD)0x80100069L)

// Access is denied due to security rules
#define SCARD_W_SECURITY_VIOLATION		((DWORD)0x8010006AL)

// PIN code is not verified, access is denied
#define SCARD_W_WRONG_CHV			((DWORD)0x8010006BL)

// Max. PIN code retry number is reached, access is denied
#define SCARD_W_CHV_BLOCKED			((DWORD)0x8010006CL)

// The last smart card file is reached, no more file available
#define SCARD_W_EOF				((DWORD)0x8010006DL)

// Operation is terminated by user
#define SCARD_W_CANCELLED_BY_USER		((DWORD)0x8010006EL)

// Smart card PIN is not set
#define SCARD_W_CARD_NOT_AUTHENTICATED		((DWORD)0x8010006FL)

#endif // SCARD_S_SUCCESS


// Extended error code  =========================================================

// File already exists
#define SCARD_E_FILE_EXISTS			((DWORD)0xA0100001L)

// Card-resident memory operation error
#define SCARD_E_EPROM_ERROR			((DWORD)0xA0100002L)

// User specified CLA is invalid
#define SCARD_E_INVALID_CLA			((DWORD)0xA0100003L)

// User specified INS is invalid
#define SCARD_E_INVALID_INS			((DWORD)0xA0100004L)

// Virtual machine address out ranged/abnormal 
#define SCARD_E_VM_ADDRESS_ERROR		((DWORD)0xA0100005L)

// 0 division error
#define SCARD_E_ZERO_DIVIDE			((DWORD)0xA0100006L)

// Card is not inserted to the proper place
#define SCARD_E_WRONG_POSITION			((DWORD)0xA0100007L)

// Card is in an unknown status
#define SCARD_E_UNKNOWN_STATE			((DWORD)0xA0100008L)

// Card is not opened
#define SCARD_E_CARD_NOT_OPENED			((DWORD)0xA0100009L)

// Unknown command
#define SCARD_E_UNKNOWN_COMMAND			((DWORD)0xA010000AL)

// Super password reset number is 0
#define SCARD_E_ZERO_TRYTIME			((DWORD)0xA010000BL)

// Too many devices are opened
#define SCARD_E_TOO_MANY_DEVICE			((DWORD)0xA010000CL)

// Invalid instruction.
#define	SCARD_E_INVALID_INSTRUCTION		((DWORD)0xA010000DL)

// card still have more data to return 
#define SCARD_W_RESPONSE			((DWORD)0xA01000FFL)

// returned data length
#define RETURN_LENGTH				((DWORD)0xA0100014)

// Error codes for virtual devices=================================================

// Virtual card file creation failure
#define SCARD_E_FILE_CREATE_FAILED		((DWORD)0xA0101001L)

// Virtual card file open failure
#define SCARD_E_FILE_OPEN_FAILED		((DWORD)0xA0101002L)

#define SCARD_E_FLOAT_NOT_FOUND         	((DWORD)0x0000e00aL)

//RSA_DES library not found
#define SCARD_E_RSADES_NOT_FOUND        	((DWORD)0x0000e00bL)

#define SCARD_E_LIBS_ERROR              	((DWORD)0x0000e00cL)

#define SCARD_E_FLOAT_NOT_FOUND         	((DWORD)0x0000e00aL)

//RSA_DES library not found
#define SCARD_E_RSADES_NOT_FOUND        	((DWORD)0x0000e00bL)

#define SCARD_E_LIBS_ERROR              	((DWORD)0x0000e00cL)

#define SCARD_E_EXTENDFLOAT_NOT_FOUND   	((DWORD)0x0000e00dL)

//Extended error codes for ROCKEY

//Cannot find ROCKEY
#define ROCKEY_K_NOT_FOUND               	((DWORD)0xF0100001L)

//Search ROCKEY failed	
#define ROCKEY_K_QUERY_FAILED            	((DWORD)0xF0100002L)

//Write ROCKEY failed
#define ROCKEY_K_WRITE_FAILED            	((DWORD)0xF0100003L)

//Read ROCKEY failed
#define ROCKEY_K_READ_FAILED            	 ((DWORD)0xF0100004L)















/***********************************************************************************************************************
						The following is new functions of Rockey6Smart Update
***********************************************************************************************************************/

//	Notes for some issues hard to understand:

// (1) Directory/file ID (2 bytes)
//     Each file or directory has an ID in the dongle. The ID is a 16-bit number. It must be unique under a directory.
//     
//     Note:
//     3F00 is a root directory ID
//     2F01 is an ATR file ID
//     3FFF is the ID of current directory (unused)
//     0000 is a reserved value
//     when creating a file or directory, the above IDs cannot be chosen
//
// (2) Directory/file class coding (1 byte)
//     a) Each file or directory has a class description byte, which is also called "class code" (FCLA).
//        It indicates the classs to which a file belongs. The files with the same class coding are treated as for the same product.
//        
//     b) The executable files in the dongle are limited to the class code (FCLA), and can only read/write to and create data files
//        with the same class code (FCLA).
//     c) The length of the class code of each file is 1 byte in IC card, ranging from 0 to FF.
//     d) There is 1 byte for the system to record the current file class, which is called "the current file class of the system"
//        (SYSFCLA).
//     e) The special value FF (hex) means "no class". The data files of FF class can be accessed by the executable files
//        of any classes.
//     f) "The current file class of the system" will be switched (SYSFCLA = FCLA) when running a "file selection" command or calling the 
//        executable files in the dongle.
//     g) If the target file to which the current file will be switched is a "no class" file, SYSFCLA will remain the class unchanged.
//     h) When a class is switched, "the current file permission of the system" will be reset to 0 (SYSPRI = 0).
//     i) By default, SYSFCLA is FF (no class) after a dongle reset.
//     j) The class of the root directory of the system is "no class".
//     k) In the dongle, files of the same class form a sub-system that is relatively independent. The data files of "no class"
//        are public data. The executable files of "no class" are public handlers.
//     l) The executable file of no class can only access the data file of no class.
//     m) The more important purpose of class coding is for the executable filter file, which will be described later.
//
//     Tips for users:
//     a) Generally, the class coding is applied to the management of developers. For example, if a developer develops several products, or 
//        if multiple developers together develop a product, class management is very useful.
//     b) If a dongle only protects a software program, it is recommended to set all class codes to FF.
//     c) It is also recommended to set the files and sub-directories under the same directory to the same class, with "file permission" for 
//        access control.
//
// (3) Directory/file permission coding (half byte)
//     a) Besides the class, each file has a "file access permission"(FPRI) for restricting access control
//        of files of the same class.
//     b) The file access permission is designed for the files of the same class. In other words, FPRI is an access control mechanism lower than 
//        FCLA.
//     c) The the length of the file access permission is half byte in the IC card, ranging from 0 to 15 (coding). There are 16 distinct permissions.
//        0 means the lowest level. 15 means the highest level.
//     d) There is 1 byte to record the current file access permission in the system: "the current file access permission of the system"
//        (SYSFPRI).
//     e) "The current file access permission of the system"(SYSFPRI) must be raised by the executable file in the dongle.
//        SYSFPRI = the permission set by the executable file via system call, but the executable file cannot set a permission over 
//        its permission: SYSFPRI <= FPRI of the executable file.
//     f) The executable file can only operate the data files with equal or lower level (compared to itself), including read, write, and create.
//     g) When logging off, the executable file can clean up "the current file access permission of the system"(SYSFPRI = 0).
//     i) For the executable files in the dongle, a file attribute FILEATTR_UPIGNORE can be configured.
//        This attribute informs the system if SYSFPRI >= the FPRI of the executable, it is not necessary
//        to call this executable file.
//
//     Tips for users:
//     a) The permission coding is a higher level of file management. If it is not required, set the permission codes to 0
//        for the files.
//     b) Note that the permission code is different from the class code: classification first, grading then.
//
// (4) Directory/file attribute (half byte)
//     Each file/directory has an attribute descriptor (FATTR). Each bit means an attribute.
//        NORMAL	0x00	// generic file					a)
//        EXEC		0x10	// executable file				b)
//        DIR		0x20	// this is a directory, instead of a file	c)
//        UPIGNORE	0x40	// Up-to-Ignore					d)
//        INTERNAL	0x80	// internal file					e)
//	   a) generic file
//     b) executable: The file is an executable inside the dongle. The file must be placed under the root directory of 
//        the dongle. It can be created or deleted only after passing super password authentication.
//     c) directory: It denotes a directory. If this property is specified, other property bits will be ignored.
//     d) Up-to-Ignore: A property bit that is valid to the executable files inside the dongle only. If SYSFPRI >= FPRI  
//        (current file security level of the system >= the security level of the executable file), the execution will be ignored.
//     e) internal file: The file can be accessed only by the executable file. The external read and write operations are 
//        not allowed. To delete the file externally, the super password is required.
//        Without supper password authentication, external access is impossible.
//
// (5) Long filename
//     A long name length of which is less than or equal to 16 bytes can be specified when creating a file or directory. 
//     The long name is optional. If a long filename is specified, 
//     the file can be accessed by the file ID or the long filename.
// ==========================================================================

// (1) The formatting operation can be performed only after passing super password authentication. 
//     When performing the operation, the user can reset the volume label and developer information 
//     (the information cannot be set at any time else).
// (2) When creating a directory, specify the ID, class, permission (security level), and long name.
// (3) The directory listing command uses an enumberation mechanism until an error is returned,
//     indicating there is no more directories or files under the current directory.
// (4) SET_CURRENT_DIR is similar to the DOS command cd xxxx. Only the directory ID or long name is required for this
//     fucntion. If a long name is used, the directory ID must be specified as 0 first. 
// (5) When creating a file, specify the file ID, class, permission, properties, file size, and long filename etc.  
//     If you do not want to specify the long filename, input an all-zero string.
// (6) The created file will be set to be the current file automatically, without calling SET_CURRENT_FILE.

#define NOT_SUPPORT_SIZE_DURING_GENERATE_RSA_KEY 0x90000001					//sizes unsupported given when generating Rsa Key (only 512, 1024-bit supported)
#define GET_3DES_KEY_FAILED	0x90000002											//failed to get 3DES key
#define NO_PASSED 0																//password is incorrect
#define SUPER_PASSWORD_PASSED 4													//super password is ok by validation
#define REMOTE_PASSWORD_PASSED 8												//remote update password is ok by validation
#define SUPER_REMOTE_PASSWORD_PASSED 12											//both super password and remote update password are ok

//	BYTE gDesKey[128];
//	BYTE gRsaPubKey[128];


	//Function:				sequential table reversion
	//for internal use
	void reverseByte(BYTE* pBuf,DWORD len);
	void reverseDword(DWORD* pBuf,DWORD len);

	//the parameter hic in the following functions is the handle of the dongle

	//Function:				get volume label and developer information
	//strVolume:			return the volume label of the dongle
	//strManufactureInfo:	return the developer informatin of the dongle
	//The 2 parameters can be changed by the developer, but the end users cannot do that
	EXTERN_C int WINAPI DIC_GetCardInfo(int hic, char strVolume[], char strManufactureInfo[]);

	//Function:				get information, such as production time, serial number, sale time, COS version
	//pdwProduceTime:		return production time of dongle
	//pdwSerial:			return serial number of dongle
	//pdwSalesTime:			return sale time of dongle
	//pdwCOSVersion:		return COS version of dongle
	//The 4 parameters are configured by the manufacturer before shipment, and cannot be changed
	EXTERN_C int WINAPI DIC_GetHardwareInfo(int hic, 
		DWORD *pdwProducedTime, 
		DWORD *pdwSerial, 
		DWORD *pdwSalesTime, 
		DWORD *pdwCOSVersion);

	//Function:					get regional code, reseller code, and 2 user codes
	//pwZoneCode:				return regional code
	//pwAgentCode:				return reseller code
	//pwUser1Code:				return code of user 1
	//pwUser2Code:				return code of user 2
	EXTERN_C int WINAPI DIC_GetManagerCode(int hic, 
		WORD *pwZoneCode, 
		WORD *pwAgentCode, 
		WORD *pwUser1Code, 
		WORD *pwUser2Code);

	//Function:					get password verification status, which is returned by enum variable piState
	//	NO_PASSED = 0,									//password not passed validation
	//	SUPER_PASSWORD_PASSED = 4,						//super password passed validation
	//	REMOTE_PASSWORD_PASSED = 8,						//remote update password passed validation
	//	SUPER_REMOTE_PASSWORD_PASSED = 12				//both super password and remote update password passed validation
	EXTERN_C int WINAPI DIC_GetPasswordVerifyState(int hic, int *piState);

	//Function:					get remote update flag and passsword
	//pdwFlag:					return remote update flag
	//abyPassword:				return remote update password
	EXTERN_C int WINAPI DIC_GetRemoteInfo(int hic, DWORD *pdwFlag, BYTE abyPassword[]);

	//Function:					set remote update flag and password
	//dwFlag:					set remote update flag
	//abyPassword:				set remote update password
	EXTERN_C int WINAPI DIC_SetRemoteInfo(int hic, DWORD dwFlag, const BYTE abyPassword[]);

	//Function:					verify remote update flag and password
	//dwFlag:					remote update flag to be verified
	//dwPassword:				remote update password to be verified
	EXTERN_C int WINAPI DIC_CheckRemoteInfo(int hic, DWORD dwFlag, const BYTE byPassword[]);

	//Below are some operations relating to file and directory

	//the class of a file or directory is a number ranging from 0x00 to 0xff

	//file and directory properties:
	//FILEATTR_NORMAL			generic data file
	//FILEATTR_EXEC				means executable
	//FILEATTR_DIR				means directory
	//FILEATTR_UPIGNORE			means Up-to-Ignore; if current file security level of the system >= security level of executable file, execution will be ignored.
	//FILEATTR_INTERNAL			means a file used by internal applications only, which cannot be operated externally; if a file is flagged as internal, it has an //implicit property; when listing directory, the implicit file can be displayed only if a super password has been validated

	//file security level is a number ranging from 0x00 to 0x0f

	//Function:					get current directory ID, class, property, security level, and name
	//pwDirID:					return current directory ID
	//pbyDirType:				return current directory class
	//pbyDirAttribute:			return current directory property
	//strDirName:				return current directory name
	EXTERN_C int WINAPI DIC_GetCurrentDir(int hic, 
		WORD *pwDirID, 
		BYTE *pbyDirType, 
		BYTE *pbyDirAttribute, 
		char strDirName[]);

	//Function:					set current directory to a directory identified by wDirID
	//wDirID:					directory ID
	//strDirName:				directory name
	//Remarks:					can configure current directory by directory ID or name; pass one of them
	//							another one can be set to 0 or NULL; if both of them are passed, they must denote the same directory
	EXTERN_C int WINAPI DIC_SetCurrentDir(int hic, 
		WORD wDirID,
		char strDirName[]);

	//Function:					get current file ID, class, property, security level, and name
	//pwFileID:					return current file ID
	//pbyFileType:				return current file class
	//pwFileSize:				return current file size
	//pbyFileAttribute:			return current file property
	//pbyFileSecurity:			return current file security level
	//strFileName:				return current file name
	EXTERN_C int WINAPI DIC_GetCurrentFile(int hic, 
		WORD *pwFileID, 
		BYTE *pbyFileType, 
		WORD *pwFileSize, 
		BYTE *pbyFileAttribute, 
		BYTE *pbyFileSecurity, 
		char strFileName[]);

	//Function:					set current file to a file identified by wFileID
	//wFileID:					file ID
	//strFileName:				file name
	//Remarks:					can be configured by file ID or name; pass one of them
	//							another one can be set to 0 or NULL; if both of them are passes, they must denote the same file
	EXTERN_C int WINAPI DIC_SetCurrentFile(int hic, 
		WORD wFileID,
		char strFileName[]);

	//Function:					get the ID, class, property, security level, and name of the upper-level directory
	//pwDirID:					return ID of the upper-level directory
	//pbyDirType:				return class of the upper-level directory
	//pbyDirAttribute:			return property of the upper-level directory
	//strDirName:				return name of the upper-level directory
	EXTERN_C int WINAPI DIC_GetParentDir(int hic, 
		WORD *pwDirID, 
		BYTE *pbyDirType, 
		BYTE *pbyDirAttribute, 
		char strDirName[]);

	//Function:					return to the upper-level directory
	EXTERN_C int WINAPI DIC_SetParentDir(int hic);

	//Function:					list all directories and files under current directory; used with DIC_FindNextFileOrDir
	//piPosition:				if -1 returned, no file/directory exists under current directory; other values mean some directories/files have not been listed yet
	//							can get remaining directories/files by calling DIC_FindNextFileOrDir function
	//pwID:						return ID of the first directory or file under current directory
	//pbyType:					return class of the first directory or file under current directory
	//pbyDirAttribute:			return property of the first directory or file under current directory
	//pbyDirSecurity:			return security level of the first directory or file under current directory (0 returned for directory search)
	//strName:					return name of the first directory or file under current directory
	EXTERN_C int WINAPI DIC_FindFirstFileOrDir(int hic, 
		int *piPosition, 
		WORD *pwID, 
		BYTE *pbyType, 
		BYTE *pbyAttribute, 
		BYTE *pbySecurity, 
		char strName[]);

	//Function:					list all directories and files under current directory; used wit DIC_FindFirstFileOrDir
	//piPosition:				if -1 returned, all directories/files under current directory have been listed; other values mean some directories/files have not //been listed yet
	//							can get remaining directories/files by calling DIC_FindNextFileOrDir function
	//pwID:						return ID of the next directory/file after piPosition under current directory
	//pbyType:					return class of the next directory/file after piPosition under current directory
	//pbyDirAttribute:			return property of the next directory/file after piPosition under current directory
	//pbyDirSecurity:			return security level of the next directory/file after piPosition under current directory (0 returned for directory search)
	//strName:					return name of the next directory/file after piPosition under current directory
	EXTERN_C int WINAPI DIC_FindNextFileOrDir(	int hic, 
		int *piPosition, 
		WORD *pwID, 
		BYTE *pbyType, 
		BYTE *pbyAttribute, 
		BYTE *pbySecurity, 
		char strName[]);

	//Note: When reading/writing to a file, the file must be set to be the current file first. This can be done by DIC_SetCurrentFile.
	//DIC_CreateFile will set the created file to be the current file automatically. 

	//Function:					read current file
	//wOffset:					offset of current file
	//pwRead:					pass the size of the data to be read; return the actual size of the data that has been read
	//pBuf:						buffer for receiving data
	EXTERN_C int WINAPI DIC_ReadFile(int hic, WORD wOffset, WORD *pwRead, char *pBuf);

	//Function:					write to current file
	//wOffset:					offset of current file
	//wWrite:					size of data to be written
	//pBuf:						buffer for storing the written data
	EXTERN_C int WINAPI DIC_WriteFile(int hic, WORD wOffset, WORD wWrite, const char *pBuf);

	//Function:					format
	//strVolume:				volume label
	//strManufactureInfo:		developer information
	//Note:					close and open to make the new format effective
	EXTERN_C int WINAPI DIC_FormatCard(int hic, char strVolume[], char strManufactureInfo[]);

	//Function:					create a directory
	//wDirID:					ID of the directory to be created
	//byDirType:				class of the directory to be created
	//byDirAttribute:			property of the directory to be created
	//strDirName:				name of the directory to be created
	EXTERN_C int WINAPI DIC_CreateDir(int hic, 
		WORD wDirID, 
		BYTE byDirType, 
		BYTE byDirAttribute, 
		const char strDirName[]);

	//Function:					create a file
	//wFileID:					ID of the file to be created
	//byFileType:				class of the file to be created
	//byFileAttribute:			property of the file to be created
	//byFileSecurity:			security level of the file to be created
	//strFileName:				name of the file to be created
	EXTERN_C int WINAPI DIC_CreateFile(int hic, 
		WORD wFileID, 
		BYTE byFileType, 
		BYTE byFileAttribute, 
		WORD wFileSize, 
		BYTE byFileSecurity, 
		const char strFileName[]);

	//Function:					delete current directory
	EXTERN_C int WINAPI DIC_DeleteDir(int hic);

	//Function:					delete current file
	EXTERN_C int WINAPI DIC_DeleteFile(int hic);

	//Function:					get a random number
	//byRandomLength:			length of the random number; the maximum is 16
	//abyRandom:				array for storing the random number
	EXTERN_C int WINAPI DIC_Random(int hic, BYTE byRandomLength, BYTE abyRandom[]);

	//Function:					run an executable file
	//wRunFileNameID:			file ID represented by the filename of executable file; not the file ID of the executable file
	//							the filename of executable file is a WORD value represented by a string, such as "6AB3" and "032B"
	//							for example, if the filename of the executable file is "6AB3", 0x6AB3 should be passed
	//wParamSize:				pass startup parameter size
	//cpcParam:					pass startup parameter
	//							the startup parameter is a string of various input parameters; for example, if the input parameters include an 8-
	//							byte double value and a 2-byte WORD value, the first 8 bytes of cpcParam is the double value and the following 2 //bytes are the WORD value; the size of wParamSize is 10
	//pwRetDataLength:			size of returned data after execution
	//pcRetData:				returned data after executaion (since the type of the returned data is unknown, users must convert it to normal byte sequence)
	//							this parameter is a string of the returned data after executaion; for example, if the returned data includes a 4- //byte float value and a 4-byte DWORD value, the 1st 4 bytes of the parameter are the float value, and the following 4 bytes are the DWORD value; the size of *pwRetDataLength is 8
								
	int WINAPI DIC_Run(int hic, WORD wRunFileNameID, WORD wParamSize, const char *cpcParam, WORD *pwRetDataLength, char *pcRetData);

	//Function:					generate a cipher-text file using TEA algorithm, which is stored in pcEncData; a temporary file is required for this //algorithm; the temporary file can be deleted by calling DIC_DeleteFile after execution of this function
	//wFileID:					ID of temporary file
	//byFileType:				class of temporary file
	//byFileAttribute:			property of temporary file
	//pwFileSize:				pass size of cpcFileData; return size of pcEncData
	//byFileSecurity:			security level of temporary file
	//strFileName:				name of temporary file
	//cpcFileData:				pointer to file data to be encrypted
	//pcEncData:				pointer to file data that has been encrypted; the data pointed (size:*pwFileSize) can be sent to destination via network media
	//							passed as the 2nd parameter of DIC_TeaEncryptedFile2PlainFile function at the destination; remote update can be 
//                                                              completed using these 2 functions                                                             
	int WINAPI DIC_EncryptFileUseTea(int hic, 
		WORD wFileID, 
		BYTE byFileType, 
		BYTE byFileAttribute, 
		WORD *pwFileSize, 
		BYTE byFileSecurity, 
		const char strFileName[], 
		const char *cpcFileData,
		char *pcEncData);

	//Function:					for remote update with DIC_EncryptFileUseTea function; write the encrypted data returned by the parameter 
	//						pcEncData of DIC_EncryptFileUseTea function to the dongle as a plain-text file; the ID is the ID of the temporary file as //                                                      defined in DIC_EncryptFileUseTea function
	//cmddata:					encrypted data returned by the parameter pcEncData of DIC_EncryptFileUseTea function
	int WINAPI DIC_TeaEncryptedFile2PlainFile(int hic, char *cmddata);

	//Function:					verify super password
	//abySuperPW:				super password; cannot be an all-zero string; or, it will be locked
	EXTERN_C int WINAPI DIC_CheckSuperPassword(int hic, BYTE abySuperPW[]);

	//Function:					set super password
	//byTryTimesCeiling:		the maximum number of retries for verification of super password
	//abySuperPW:				super password; cannot be an all-zero string; or, it will be locked
	EXTERN_C int WINAPI DIC_SetSuperPassword(int hic, BYTE byTryTimesCeiling, BYTE abySuperPW[]);

	//Function:					get remote update password
	//dwRemoteFlag:				flag of this remote update
	//dwHardSerial:				hardware serial number of this remote update
	//abyRemotePW:				enter the password of this remote update; return the password of next remote update
	EXTERN_C int WINAPI DIC_GetRemoteUpgradePassword(int hic, DWORD dwRemoteFlag, DWORD dwHardSerial, BYTE abyRemotePW[]);

	//						for DES algorithm, the length of the encrypted/decrypted data must be an integer number of times of  key length (8 bytes)

	//Function:					encrypt a set of data using DES algorithm
	//wKeyFileID:				ID of key file
	//byKeyIndex:				key index in key file; indicates which key is used
	//wDataLength:				length of data to be encrypted
	//cpcDataSrc:				data to be encrypted
	//pcDataEnc:				data that has been encrypted
	EXTERN_C int WINAPI DIC_DesEncrypt(int hic, WORD wKeyFileID, BYTE byKeyIndex, WORD wDataLength, const char *cpcDataSrc, char *pcDataEnc);

	//Function:					decrypt using DES algorithm
	//wKeyFileID:				ID of key file
	//byKeyIndex:				key index in key file; see description of DIC_GenerateDesKeyFile function
	//wDataLength:				length of data to be decrypted
	//cpcDataSrc:				data to be decrypted
	//pcDataDec:				data that has been decrypted
	EXTERN_C int WINAPI DIC_DesDecrypt(int hic, WORD wKeyFileID, BYTE byKeyIndex, WORD wDataLength, const char *cpcDataSrc, char *pcDataDec);

	//Function:					generate a DES key file; length of key can be 8 or 16 bytes; for 16-byte keys, 3DES algorithm will be used automatically //                                                      for encryption/decryption
	//wKeyFileID:				ID of the key file that is generated
	//wKeyDataLength:			length of key data
	//pcKeyData:				key data
	//							the key data has a form like the following: "\x03\x10\x01\x02\x03\x04\x05\x06\x07\x08\x09//        \x0A\x0B\x0C\x0D\x0E\x0F\x10\
	//\x01\x08\x01\x02\x03\x04\x05\x06\x07\x08"	where, the first \x03 means the following is a key indexed as 3; the second \x10 (hex) means the length of the key is 16 
//                                                      bytes; the following 16 bytes are the key indexed as 3; 
//                                                      the following 19th byte \x01 means the following is a key indexed as 1; 
//                                                      \x08 means the length of the key is 8 bytes; the following 8 bytes is the key indexed as 1;

	//							users can write keys to the key file by calling DIC_WriteFile function
	//							for example, to generate a DES key file containing the above key data and then write a key indexed as 2 which is 8 //                                                              bytes long, the code can be as follows:
	//							DIC_GenerateDesKeyFile(hic, 0x1003, 28, "\x03\x10\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F\x10\
	//\x01\x08\x01\x02\x03\x04\x05\x06\x07\x08");
	//							DIC_SetCurrentFile(hic, 0x1003);
	//							DIC_WriteFile(hic, 28, 10, "\x02\x08\x01\x02\x03\x04\x05\x06\x07\x08");
	EXTERN_C int WINAPI DIC_GenerateDesKeyFile(int hic, WORD wKeyFileID, WORD wKeyDataLength, char *pcKeyData);

	//Function:					generate RSA public/private key file using specified public/private key ID and size of public key; currently, only 512-bit //                                                      and 1024-bit are supported as key length
	//wPublicKeyFileID:			ID of public key file
	//wPublicKeySize:			size of public key
	//wPrivateKeyFileID:		ID of private key file
	EXTERN_C int WINAPI DIC_GenerateRsaKey(int hic, WORD wPublicKeyFileID, WORD wPublicKeySize, WORD wPrivateKeyFileID);

	//							for RSA algorithm, the following must be known: decrypt a set of plain data with a private key and then encrypt it //                                                              with a public keyencrypt the plain data with a public key and then decrypt it with a private keyplain data
	//							length of the data to be encrypted/decrypted must be an integer number of times of key length 512/8=64 or //1024/8=128

	//Function:					encrypt a set of data using RSA algorithm
	//wPublicKeyFileID:			ID of public key file
	//wDataLength:				length of data to be encrypted
	//cpcDataSrc:				data to be encrypted
	//pcDataEnc:				data that has been encrypted
	//bIsDataSrcFromOpenssl:	if the data to be encrypted is from the data that has been dealed with by Openssl or not (if so, the byte sequence must be reversed before //                                      the dongle)
	EXTERN_C int WINAPI DIC_RsaEncrypt(int hic, WORD wPublicKeyFileID, WORD wDataLength, const char *cpcDataSrc, char *pcDataEnc, BOOL bIsDataSrcFromOpenssl);


	//Function:					decrypt a set of data using RSA algorithm
	//wPrivateKeyFileID:		ID of private key file
	//wDataLength:				length of data to be decrypted
	//cpcDataSrc:				data to be decrypted
	//pcDataDec:				data that has been decrypted
	//bIsDataSrcFromOpenssl:	if the decrypted data is from the data that has been dealed with by Openssl (if so, the byte sequence must be reversed before the dongle)
	EXTERN_C int WINAPI DIC_RsaDecrypt(int hic, WORD wPrivateKeyFileID, WORD wDataLength, const char *cpcDataSrc, char *pcDataDec, BOOL bIsDataSrcFromOpenssl);

	//Function:					get size of remaining dongle space
	//pdwFreeSpace:				return the size of the remaining dongle space
	EXTERN_C int WINAPI DIC_GetFreeSpace(int hic, DWORD *pdwFreeSpace);

	//Function:					set the initial value of counter
	//dwInit:					the initial value of the counter
	EXTERN_C int WINAPI DIC_SetCounter(int hic, DWORD dwInit);

	//Function:					decrease the value of the counter by 1
	EXTERN_C int WINAPI DIC_DecreaseCounter(int hic);

	//Function:					generate a public key file with Openssl
	//wPublicKeyFileID			ID of public key file
	//wPublicKeySize			size of public key
	//cpbyPublicKeyData			public key data (typically, it is "n" in the RSA structure resulted by BN_bn2bin function)
	EXTERN_C int WINAPI DIC_GenerateRsaPublicKeyFile(int hic, WORD wPublicKeyFileID, WORD wPublicKeySize/*512/1024*/, const BYTE *cpbyPublicKeyData);

	//Function:					generate a private key file with Openssl
	//wPrivateKeyFileID			ID of private key file
	//wPrivateKeySize			size of private key
	//cpbyPrivateKeyDataD		private key data D (typically, it is "d" in the RSA structure resulted by BN_bn2bin function)
	//cpbyPrivateKeyDataN		private key data N (typically, it is n in the RSA structure resulted by BN_bn2bin function)
	EXTERN_C int WINAPI DIC_GenerateRsaPrivateKeyFile(int hic, WORD wPrivateKeyFileID, WORD wPrivateKeySize/*512/1024*/, const BYTE *cpbyPrivateKeyDataD, const BYTE *cpbyPrivateKeyDataN);

#endif //__DIC32_H__
