// Command.cpp: implementation of the CCommand class.
//
//////////////////////////////////////////////////////////////////////

#include "command.h"

#include "DICNet.h"


#include "stdio.h"
#include "conio.h"


extern char g_Ver[];
static char chHelpInfo[]="show all commands,or show details of specified command.\n";



static void GetPath(WORD* pPath,char* pBuf)
{
	char chTemp[16];
	WORD wLen=pPath[0];

	pBuf[0]=0;

	for(WORD i=1; i<=wLen; i++)
	{
		sprintf(chTemp,"\\%04X",pPath[i]);
		strcat(pBuf,chTemp);
	}
}

static	void ShowFileTitle()
{
	const char chFormat1[]="%-20s%-8s%-8s%-8s%-8s\n";

	printf(chFormat1,"File name","ID","Size","Class","Attr");
}

static void ShowFileInfo(DICST_File* pfi)
{
	if (pfi->fileid == 0) {
		return;
	}
	const char chFormat2[]="%-20s%04X    %-8u%02X      %s\n";
	char  chName[32],chAtr[32];

	//Get name, if it's dir, add []
	if(pfi->fileatrpri&FILEATTR_DIR) sprintf(chName,"[%s]",pfi->filename);
	else strcpy(chName,pfi->filename);
	
	//Get attribute string.
	sprintf(chAtr,"%02X ",pfi->fileatrpri);
	
	if(pfi->fileatrpri&0xF0)
	{
		strcat(chAtr,"( ");
		if(pfi->fileatrpri&FILEATTR_DIR)		strcat(chAtr,"D ");
		if(pfi->fileatrpri&FILEATTR_EXEC)		strcat(chAtr,"E ");
		if(pfi->fileatrpri&FILEATTR_UPIGNORE)	strcat(chAtr,"U ");
		if(pfi->fileatrpri&FILEATTR_INTERNAL)	strcat(chAtr,"I ");
		strcat(chAtr,")");
	}
	
	printf(chFormat2,chName,pfi->fileid,pfi->filesize,pfi->filecla,chAtr);		

}


//Cmd Define.
static EDEN_CMD g_cmd[]=
{
	{"?"		,(DWORD)OnHelp,			chHelpInfo},
	{"Help"		,(DWORD)OnHelp,			chHelpInfo},
	{"Ver"		,(DWORD)DoVer,			"Show version info."},
	{"Exit"		,(DWORD)0,				"Exit program."},
	{"Find"		,(DWORD)DoFind,			"Find NetROCKEY6 via LAN.\nUsage:\n(1)Find , find All NetROCKEY6s.\n(2)Find Zone Agent User1 User2, find specified NetROCKEY6.(Input are Hex)\n"},
	{"LR"		,(DWORD)DoList,			"List all found NetROCKEY6s .\n"},
	{"Query"	,(DWORD)DoQuery,		"Query informaton. Usage: Query index\n"},
	{"Open"		,(DWORD)DoOpen,			"Open NetROCKEY6. Usage: Open index module\n"},
	{"Close"	,(DWORD)DoClose,		"Close current selected handle.\n"},
	{"LH"		,(DWORD)DoListHandle,	"List all opened handles.\n"},
	{"SH"		,(DWORD)DoSettHandle,	"Set Current handle. Usage: SH handle, such as SH 101\n"},
	{"GCInfo"	,(DWORD)DoGetCard,		"Get ATR and Volume information of current selected card.\n"},
	{"GHInfo"	,(DWORD)DoGetHWInfo,	"Get hardware information of current selected card.\n"},
	{"GMCode"	,(DWORD)DoGetMCode,		"Get Manager code.\n"},
	{"GRInfo"	,(DWORD)GetRomoteInfo,	"Get Remote information.\n"},
	{"SRInfo"	,(DWORD)SetRomoteInfo,	"Init Remote information, do it after format. Usage: srinfo RemoteTag RemotePass, such as:\nsrinfo 7A1B32AC FF 02 FF 03 FF 04 FF 05\n"},
	{"CRInfo"	,(DWORD)ChkRomoteInfo,	"Chcek Remote password. Call it after lock. Usage: crinfo RemoteTag RemotePass, such as:\ncrinfo 7A1B32AC FF 02 FF 03 FF 04 FF 05\n"},
	{"Prvg"		,(DWORD)Prvg,			"Get card privilege.\n"},
	{"Dir"		,(DWORD)Dir,			"List all files in current directory.\n"},
	{"cd"		,(DWORD)DoCD,			"Enter sub directory. Usage: cd DirID, such as cd 03a5.\n"},
	{"md"		,(DWORD)DoMD,			"Create a sub-directory. Usage: md DirID class attr [name], such as:\nmd 1a2c ff 00 testdir.\n"},
	{"rd"		,(DWORD)DoRD,			"Delete current directory.\n"},
	{"select"	,(DWORD)Select,			"Select a file or Show current dir and file. Usage:\n(1) select fileID to select a file.\n(2) select to show current selection.\n"},
	{"create"	,(DWORD)Create,			"Create file. Usage: create FileID size class attr [name], such as:\ncreate 1a2c 100 ff 00 testdir. (size is decimal)\n"},
	{"delete"	,(DWORD)Delete,			"Delete current selected file.\n"},
	{"Read"		,(DWORD)DoRead,			"Read selected file. Usage: read offset size. such as:\nRead 100 20  (Both offset and size are decimal)\n"},
	{"Write"	,(DWORD)DoWrite,		"Write random data to selected file. Usage: write offset size. such as:\nWrite 100 20  (Both offset and size are decimal)\n"},
	{"run"		,(DWORD)Run,			"Run application. Usage:\n\t(1)run Name size data. such as: Run 2F04 4 08 00 06 00  (size is decimal)\n\t(2)Run Name R size, such as Run 2F04 R 152, input 152 random numbers\n"},
	{"Lock"		,(DWORD)DoLock,			"Lock NetROCKEY6. You can do serially operations after lock. The whole NetROCKEY6 can only be used by this handle.\n"},
	{"Unlock"	,(DWORD)DoUnlock,		"Unlock NetROCKEY6.\n"},
	
	
};

static	int	g_iMaxDevice=0;




static	CDICNet	g_DICNet5;	
	


static	DWORD*	g_arrHandle=NULL;
static	DWORD	g_dwMaxHandle=0;
static	DWORD	g_dwCurHandle=-1;

////////////////////////////////////////////////
//
//	static function.
//
///////////////////////////////////////////////

//for Init
static	void	InitCmd();
static	void	FinalRlsCmd();

//For Handle Mangaer.
static int	FindHandle(DWORD dwHandle);
static void DeleteHandle(int iIndex);


//Init Global varible.
void InitCmd()
{
	g_arrHandle=(DWORD*)malloc(1024*sizeof(DWORD));
	g_dwMaxHandle=0;
}

void FinalRlsCmd()
{
	//close all Handle.
	for(DWORD i=0; i<g_dwMaxHandle; i++)
	{
		printf("Closing handle %04x ... ",g_arrHandle[i]);
		g_DICNet5.Close(g_arrHandle[i]);
		printf(" [ok]\n");
	}

	//release alloc.
	if(g_arrHandle) free(g_arrHandle);
}


void parse_command()
{	
	char 	chPool[1024];
	char 	chCmd[256]={0};
	int 	i;
	
	CMDFUNC	pCmdFunc;

	//Init global varible.
	InitCmd();
	
	//Show Command.
	OnHelp(chCmd);
	
	
	while(1)
	{
		//Show input sign. if selected handle,show it in left.
		if(-1==g_dwCurHandle) printf("]");
		else 
		{
			int	 iTempRet;
			WORD wPath[128];

			iTempRet=g_DICNet5.Command(g_dwCurHandle,NET_GETPATH,wPath);
			if(iTempRet) wPath[0]=0;			
			
			GetPath(wPath,chPool);
			printf("%04X%s]",g_dwCurHandle,chPool);
			
			
		}
	
		chCmd[0]=0;
		if(!gets(chPool)) continue;
		sscanf(chPool,"%s",chCmd);
		if(!chCmd[0]) continue;

		if(!stricmp(chCmd,"exit")) break;
		
		pCmdFunc=NULL;		
		for(i=0; i<sizeof(g_cmd)/sizeof(EDEN_CMD); i++)
		{
			if(!strnicmp(g_cmd[i].chCmd,chCmd,strlen(g_cmd[i].chCmd)))
			{
				pCmdFunc=(CMDFUNC)g_cmd[i].dwFuncAddr;
				break;
			}
		}
		
		if(pCmdFunc) pCmdFunc(chPool);
		else printf("Unkown command.\n");
	}

	FinalRlsCmd();
}


void OnHelp(char *cmdline)
{
	char 	buf[1024];
	char*	helpinfo=NULL;
	int	i=0;	
	
	if(sscanf(cmdline,"%s %s",buf,buf)!=2)
	{
		//show all command.
		while(i<sizeof(g_cmd)/sizeof(EDEN_CMD))
		{		
			printf(g_cmd[i].chCmd);
			i++;
			if(i%5) printf("\t\t");
			else printf("\n");
		}
		printf("\n");
	}
	else
	{
		//show specified command.
		for(i=0; i<sizeof(g_cmd)/sizeof(EDEN_CMD); i++)
		{
			if(!stricmp(g_cmd[i].chCmd,buf))
			{
				helpinfo=g_cmd[i].chHelp;
				break;
			}
		}
		
		if(helpinfo) printf(helpinfo);
		else printf("Unkown command.\n");
	}	
}

int CheckDll()
{
	return g_DICNet5.m_bInitOK;
}


void DoVer()
{
	printf("******** Nr6Test, ver %s\n",g_Ver);
}


void DoFind(char *cmdLine)
{
	char	chSvrName[MAX_PATH];
	char	chTemp[1024];
	DWORD	dwHID;
	DWORD	dwZone,dwAgent,dwUser1,dwUser2;
	int		iRet;

	iRet=sscanf(cmdLine,"%s %x %x %x %x",chTemp,&dwZone,&dwAgent,&dwUser1,&dwUser2);

	if(iRet==1)
	{
		dwZone=dwAgent=dwUser1=dwUser2=0;
	}
	else if(iRet!=5)
	{
		OnHelp("? Find");
		return;
	}


	g_iMaxDevice=g_DICNet5.Find((WORD)dwZone,(WORD)dwAgent,(WORD)dwUser1,(WORD)dwUser2);

	for(int i=0; i<g_iMaxDevice; i++)
	{
		g_DICNet5.Query(i,NR_QUERY_SVRNAME,chSvrName);
		g_DICNet5.Query(i,NR_QUERY_HID,&dwHID);

		printf("(%02d) HID: %08x,  SvrName: %s\n",i,dwHID,chSvrName);
	}

	printf("NetROCKEY6 Count:%d\n",g_iMaxDevice);
}


void DoQuery(char* cmdLine)
{
	DWORD	dwIndex;
	char	chBuf[1024];
	int		iRet;
	DWORD	dwHID;

	DICST_CardInfo cardInfo;

	if(2!=sscanf(cmdLine,"%s %d ",chBuf,&dwIndex))
	{
		OnHelp("? Query");
		return;
	}

	if(dwIndex>=(DWORD)g_iMaxDevice)
	{
		printf("****** No such index .(dwIndex<%d)\n",g_iMaxDevice);
		return;
	}

	//Query server Name.
	iRet=g_DICNet5.Query(dwIndex,NR_QUERY_SVRNAME,chBuf);
	if(iRet)
	{
		//Query failed.
		printf("****** Query server name of %d failed,code %08x(%u)\n",dwIndex,iRet,iRet);		
		return;
	}

	printf("***** Server Name: %s\n",chBuf);

	//Query HID.
	iRet=g_DICNet5.Query(dwIndex,NR_QUERY_HID,&dwHID);
	if(iRet)
	{
		//Query failed.
		printf("****** Query HID of %d failed,code %08x(%u)\n",dwIndex,iRet,iRet);		
		return;
	}

	printf("***** HID        : %08X\n",dwHID);

	//Query Card Info.
	iRet=g_DICNet5.Query(dwIndex,NR_QUERY_CARDINFO,&cardInfo);
	if(iRet)
	{
		//Query failed.
		printf("****** Query card information of %d failed,code %08x(%u)\n",dwIndex,iRet,iRet);		
		return;
	}
	
	

	memcpy(chBuf,cardInfo.atr,15);
	chBuf[15]=0;
	printf("***** ATR        : %s\n",chBuf);

	memcpy(chBuf,cardInfo.volume,16);
	chBuf[16]=0;
	printf("***** Volume     : %s\n",chBuf);




}


void DoOpen(char *cmdLine)
{
	DWORD	dwIndex,dwModule;
	char	chBuf[1024];
	int		iRet;

	if(3!=sscanf(cmdLine,"%s %d %d",chBuf,&dwIndex,&dwModule))
	{
		OnHelp("? Open");
		return;
	}

	if(dwIndex>=(DWORD)g_iMaxDevice||dwModule>256)
	{
		printf("****** No such index or module.(dwIndex<%d,dwModule<%d)\n",g_iMaxDevice,256);
		return;
	}

	iRet=g_DICNet5.Open(dwIndex,dwModule,&g_arrHandle[g_dwMaxHandle]);

	if(iRet)
	{
		//open failed.
		printf("****** Open %d,%d failed,code %08x(%u)\n",dwIndex,dwModule,iRet,iRet);		
	}
	else
	{
		//succeeded.
		g_dwCurHandle=g_arrHandle[g_dwMaxHandle];
		printf("****** Open %d,%d Succeeded,handle %04x\n",dwIndex,dwModule,g_dwCurHandle);				
		g_dwMaxHandle++;
	}	
	
}

void DoClose()
{
	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//find Index.
	int	iIndex;

	iIndex=FindHandle(g_dwCurHandle);
	if(iIndex==-1)
	{
		//Handle not found.
		printf("****** Handle %04X Not found.\n",g_dwCurHandle);						
	}
	else
	{
		//ok, close it.
		printf("Closing handle %04X ... ",g_dwCurHandle);
		g_DICNet5.Close(g_dwCurHandle);
		printf(" [ok]\n");

		DeleteHandle(iIndex);		
	}	

	//Set a new handle.
	if(g_dwMaxHandle==0) g_dwCurHandle=-1;	
	else g_dwCurHandle=g_arrHandle[g_dwMaxHandle-1];
}


//Return Index of handle.
//return -1 means not in array.
int FindHandle(DWORD dwHandle)
{
	int iRet=-1;

	for(int i=0; i<(int)g_dwMaxHandle; i++)
	{
		if(g_arrHandle[i]==dwHandle) 
		{
			iRet=i;	break;
		}
	}

	return iRet;
}

//Delete a handle.
void DeleteHandle(int iIndex)
{
	if(iIndex>=(int)g_dwMaxHandle) return;

	memcpy(&g_arrHandle[iIndex],&g_arrHandle[iIndex+1],sizeof(DWORD)*(g_dwMaxHandle-iIndex-1));

	g_dwMaxHandle--;
}


//Get card information
void DoGetCard()
{
	DWORD			dwRet;
	DICST_CardInfo	ci;

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	dwRet=g_DICNet5.Command(g_dwCurHandle,GET_CARD_INFO,&ci);

	if(dwRet)
	{
		//No Selected.
		printf("****** GetCardInfo failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		return;
	}

	printf("****** ATR   : %s\n****** VOLUME: %s\n",ci.atr,ci.volume);


}

void DoListHandle()
{
	if(0==g_dwMaxHandle)
	{
		//No handle
		printf("****** No handle opened. Please use Open command to get a handle.\n");				
		return;
	}	


	for(int i=0; i<(int)g_dwMaxHandle; i++)
	{
		printf("(%d) Handle: %04X\n",i,g_arrHandle[i]);				
	}

	printf("Totle %d Handles opened. ",g_dwMaxHandle);				

	if(-1!=g_dwCurHandle)
		printf("Current selected handle: %04X.\n",g_dwCurHandle);
	else 
		printf("\n");
}

void DoSettHandle(char* cmdLine)
{
	DWORD	dwHandle;
	char	chBuf[1024];

	//Check input format.
	if(2!=sscanf(cmdLine,"%s %x",chBuf,&dwHandle))
	{
		OnHelp("? SH");
		return;
	}

	//Check if that handle exists.
	int	iIndex;

	iIndex=FindHandle(dwHandle);
	if(iIndex==-1)
	{
		//Handle not found.
		printf("****** Handle %04X Not opened.\n",dwHandle);						
	}
	else
		g_dwCurHandle=dwHandle;

}

//List All Device
void DoList()
{
	char	chSvrName[MAX_PATH];
	DWORD	dwHID;	

	for(int i=0; i<g_iMaxDevice; i++)
	{
		g_DICNet5.Query(i,NR_QUERY_SVRNAME,chSvrName);
		g_DICNet5.Query(i,NR_QUERY_HID,&dwHID);

		printf("(%02d) HID: %08x,  SvrName: %s\n",i,dwHID,chSvrName);
	}

	printf("NetROCKEY6 Count:%d\n",g_iMaxDevice);
}



//Get Hardware info
void DoGetHWInfo()
{
	DWORD			dwRet;
	DICST_HardInfo	hi;

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	dwRet=g_DICNet5.Command(g_dwCurHandle,GET_HARDWARE_INFO,&hi);

	if(dwRet)
	{
		//No Selected.
		printf("****** GetHardwareInfo failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		return;
	}

	printf("****** COS Version : %c%c.%c%c\n****** Factory Time: %08X\n****** Hard Serial : %08X\n****** Ship Time   : %08X\n",
		hi.COSVersion>>24,hi.COSVersion>>16,hi.COSVersion>>8,hi.COSVersion,hi.FactoryTime,hi.HardSerial,hi.ShipTime);
}


//List files 
void Dir()
{
	DWORD			dwRet;
	DICST_File*		pfi;
	int				iCount=0;
	int				iIndex=0;
	
	
	char			chBuf[256];
	

	
	pfi=(DICST_File*)chBuf;

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//List All files.
	printf("----------------------------------------------------------------------\n");

	ShowFileTitle();
	
	printf("----------------------------------------------------------------------\n");

	while(1)
	{
		chBuf[0]=(BYTE)iIndex;

		
		dwRet=g_DICNet5.Command(g_dwCurHandle,LIST_DIR,chBuf);
		if(dwRet) 
		{
			if(dwRet!=SCARD_W_EOF) 	printf("****** List_DIR failed. code %08X.\n",dwRet);				
			break;
		}

		ShowFileInfo(pfi);

		
		iIndex++;

	}	

	printf("----------------------------------------------------------------------\n");

	
}

void DoCD(LPCTSTR cmdLine)
{
	//Check input format.
	char	chBuf[1024];
	DWORD	dwID;
	DWORD	dwRet;

	DICST_File	fileInfo;


	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	


	//Check if it's cd..
	char* pTemp=(char*)cmdLine+2;

	while(*pTemp && *pTemp==' ') pTemp++;
	
	if(!strcmp(pTemp,".."))
	{
		// cd..
		dwRet=g_DICNet5.Command(g_dwCurHandle,SET_PARENT_DIR,chBuf);
		if(dwRet)
		{
			printf("****** Set_Parent_Dir, retCode=%08X(%u).\n",dwRet,dwRet);				
		}	

		return;
	}


	

	if(2!=sscanf(cmdLine,"%s %x",chBuf,&dwID))
	{
		OnHelp("? cd");
		return;
	}

	
	ZeroMemory(&fileInfo,sizeof(DICST_File));
	fileInfo.fileid=(WORD)dwID;

	//DO cd
	dwRet=g_DICNet5.Command(g_dwCurHandle,SET_CURRENT_FILE,&fileInfo);

	if(dwRet)
	{
		printf("****** CD failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}	
}


void DoMD(LPCTSTR cmdLine)
{
	DWORD	dwID,dwAttr,dwClass;
	char	chBuf[1024];
	char	chName[256];
	int		iRead;
	DWORD	dwRet;

	DICST_File fl;

	//Get data form command line.
	iRead=sscanf(cmdLine,"%s %X %x %x %s",chBuf,&dwID,&dwClass,&dwAttr,chName);

	if(iRead!=5&&iRead!=4)
	{
		OnHelp("? md");
		return;
	}

	if(iRead==4) chName[0]=0;

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	fl.fileatrpri=((BYTE)dwAttr|FILEATTR_DIR);
	fl.filecla=(BYTE)dwClass;
	fl.fileid=(WORD)dwID;
	fl.filesize=0;
	strncpy(fl.filename,chName,15);

	//List All files.
	printf("----------------------------------------------------------------------\n");

	ShowFileTitle();
	ShowFileInfo(&fl);

	printf("----------------------------------------------------------------------\n");

	printf("Create this one? (Y/N)");

	char	ch;
	int		bCreate=0;

	while(1)
	{
		ch=getch();
		if(ch=='y' || ch=='Y' ) 
		{
			bCreate=1; break;
		}

		if(ch=='N' || ch== 'n') break;
	}
	
	printf("\n");

	if(!bCreate) return;

	//DO cd
	dwRet=g_DICNet5.Command(g_dwCurHandle,CREATE_DIR,&fl);

	if(dwRet)
	{
		printf("****** md failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}	
}

//remove dir.
void DoRD(LPCTSTR cmdLine)
{	
	DWORD	dwRet;

	DICST_File fl;
	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO rd.	
	ZeroMemory(&fl,sizeof(fl));
	

	dwRet=g_DICNet5.Command(g_dwCurHandle,REMOVE_DIR,&fl);

	if(dwRet)
	{
		printf("****** Delete current directory failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}	
}


void DoGetMCode()
{
	DICST_ManagerCode	dm;
	DWORD				dwRet;	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO GetManagerCode	

	dwRet=g_DICNet5.Command(g_dwCurHandle,GET_MANAGER_CODE,&dm);

	if(dwRet)
	{
		printf("****** Get_Manager_Code failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}
	else
	{
		printf("Zone: %04X, Agent: %04X, User1: %04X, User2: %04X\n",dm.Zone,dm.Agent,dm.User1,dm.User2);
	}
}

void Prvg()
{
	BYTE	byBuf[256];
	DWORD	dwRet;	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO GetManagerCode	
	dwRet=g_DICNet5.Command(g_dwCurHandle,GET_CARD_PRIVILEGE,byBuf);

	if(dwRet)
	{
		printf("****** Get_Card_Privilege failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}
	else
	{
		printf("Current Privilege: %02X ( ",byBuf[0]);

		if(byBuf[0]==0) printf("DICPR_NOPASS ");
		else
		{
			if(byBuf[0]&1) printf("AGENTPASS ");
			if(byBuf[0]&2) printf("MANUFACTPASS ");
			if(byBuf[0]&4) printf("SUPERPASS ");
			if(byBuf[0]&8) printf("REMOTEPASS ");
		}

		printf(")\n");
	}
}

void GetRomoteInfo()
{
	DICST_RemoteInfo	rm;
	DWORD				dwRet;	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO GetManagerCode	

	dwRet=g_DICNet5.Command(g_dwCurHandle,GET_REMOTE_INFO,&rm);

	if(dwRet)
	{
		printf("****** Get_Remote_Info failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}
	else
	{
		printf("RemoteTag: %08X, RemotePass: %02X %02X %02X %02X %02X %02X %02X %02X\n",
			rm.RemoteTag,
			rm.RemotePass[0],rm.RemotePass[1],rm.RemotePass[2],rm.RemotePass[3],
			rm.RemotePass[4],rm.RemotePass[5],rm.RemotePass[6],rm.RemotePass[7]);
	}
}

void SetRomoteInfo(LPCTSTR cmdLine)
{
	char				chBuf[1024];
	DWORD				dwPW[8];
	DICST_RemoteInfo	rm;
	DWORD				dwRet;	
	int					i;

	//Get data.
	dwRet=sscanf(cmdLine,"%s %x %x %x %x %x %x %x %x %x",&chBuf,&rm.RemoteTag,
		&dwPW[0],&dwPW[1],&dwPW[2],&dwPW[3],
		&dwPW[4],&dwPW[5],&dwPW[6],&dwPW[7]);

	if(dwRet!=10)
	{
		OnHelp("? SRInfo");
		return;
	}

	for(i=0; i<8; i++) rm.RemotePass[i]=(BYTE)dwPW[i];


	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO GetManagerCode	

	dwRet=g_DICNet5.Command(g_dwCurHandle,SET_REMOTE_INFO,&rm);

	if(dwRet)
	{
		printf("****** Set_Remote_Info failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}
	else
	{
		printf("Succeeded.\n");
	}
}

void ChkRomoteInfo(LPCTSTR cmdLine)
{
	char				chBuf[1024];
	DWORD				dwPW[8];
	DICST_RemoteInfo	rm;
	DWORD				dwRet;	
	int					i;	

	//Get data.
	dwRet=sscanf(cmdLine,"%s %x %x %x %x %x %x %x %x %x",&chBuf,&rm.RemoteTag,
		&dwPW[0],&dwPW[1],&dwPW[2],&dwPW[3],
		&dwPW[4],&dwPW[5],&dwPW[6],&dwPW[7]);

	if(dwRet!=10)
	{
		OnHelp("? CRInfo");
		return;
	}

	for(i=0; i<8; i++) rm.RemotePass[i]=(BYTE)dwPW[i];


	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//DO GetManagerCode	

	dwRet=g_DICNet5.Command(g_dwCurHandle,CHECK_REMOTE_INFO,&rm);

	if(dwRet)
	{
		printf("****** Check_Remote_Info failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}
	else
	{
		printf("Succeeded. You can do privilege operations now.\n");
	}
}

void Select(LPCTSTR cmdLine)
{
	char				chBuf[1024];
	DICST_File			fi;	
	DWORD				dwRet,dwID;	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	//Get data.
	dwRet=sscanf(cmdLine,"%s %x",&chBuf,&dwID);
	
	if(dwRet==2)
	{
		//Select ID.
		ZeroMemory(&fi,sizeof(&fi));
		fi.fileid=(WORD)dwID;

		dwRet=g_DICNet5.Command(g_dwCurHandle,SET_CURRENT_FILE,&fi);

		if(dwRet)
		{
			printf("****** Set File %04X failed, retCode=%08X(%u).\n",dwID,dwRet,dwRet);							
		}	


	}
	else
	{
		//Show Current Selected.
		printf("Current Selected Dir and file:\n");
		
		//List All files.
		printf("----------------------------------------------------------------------\n");
		
		ShowFileTitle();
		
		printf("----------------------------------------------------------------------\n");

		dwRet=g_DICNet5.Command(g_dwCurHandle,GET_CURRENT_DIR,&fi);

		if(dwRet)
		{
			printf("****** Get_Current_Dir failed, retCode=%08X(%u).\n",dwRet,dwRet);				
			return;
		}

		ShowFileInfo(&fi);

		dwRet=g_DICNet5.Command(g_dwCurHandle,GET_CURRENT_FILE,&fi);

		if(dwRet)
		{
			printf("****** No file have been selected, retCode=%08X(%u).\n",dwRet,dwRet);				
			return;
		}

		ShowFileInfo(&fi);

		printf("----------------------------------------------------------------------\n");

		printf("use Select fileID to select a file.\n");
		
	}	
}

void Create(LPCTSTR cmdLine)
{
	DWORD	dwID,dwAttr,dwClass,dwSize;
	char	chBuf[1024];
	char	chName[256];
	int		iRead;
	DWORD	dwRet;

	DICST_File fl;

	//Get data form command line.
	iRead=sscanf(cmdLine,"%s %X %d %x %x %s",chBuf,&dwID,&dwSize,&dwClass,&dwAttr,chName);

	if(iRead!=6&&iRead!=5)
	{
		OnHelp("? create");
		return;
	}

	if(iRead==5) chName[0]=0;

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}	

	if(dwSize>32768)
	{
		printf("****** Size is too large.\n");				
		return;
	}

	fl.fileatrpri=(BYTE)dwAttr;
	fl.filecla=(BYTE)dwClass;
	fl.fileid=(WORD)dwID;
	fl.filesize=(WORD)dwSize;
	strncpy(fl.filename,chName,16);

	//List All files.
	printf("----------------------------------------------------------------------\n");

	ShowFileTitle();
	ShowFileInfo(&fl);

	printf("----------------------------------------------------------------------\n");

	printf("Create this one? (Y/N)");

	char	ch;
	int		bCreate=0;

	while(1)
	{
		ch=getch();
		if(ch=='y' || ch=='Y' ) 
		{
			bCreate=1; break;
		}

		if(ch=='N' || ch== 'n') break;
	}
	
	printf("\n");

	if(!bCreate) return;

	//DO Create file
	dwRet=g_DICNet5.Command(g_dwCurHandle,CREATE_FILE,&fl);

	if(dwRet)
	{
		printf("****** create file failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}	
}

//Delet selected file.

void Delete()
{
	DWORD	dwRet;
	BYTE	byBuf[256];
	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}		

	dwRet=g_DICNet5.Command(g_dwCurHandle,REMOVE_FILE,byBuf);

	if(dwRet)
	{
		printf("****** Delete current file failed, retCode=%08X(%u).\n",dwRet,dwRet);				
	}	
}


static void DumpBuf(BYTE *Buf, int len)
{
	char ShowBuf[4096];
	char tempbuf[10];

	ShowBuf[0]=0;
	for(int i=0; i<len; i++)
	{
		if(i%16==0&&i)
		{
			printf(ShowBuf);
			printf("\n");
			ShowBuf[0]=0;
		}

		sprintf(tempbuf,"%02hx ",(unsigned char)Buf[i]);
		lstrcat(ShowBuf,tempbuf);		
	}
	lstrcat(ShowBuf,"\n");
	
	printf(ShowBuf);
}




void Run(LPCTSTR cmdLine)
{
	char	chBuf[1024];
	char	chFmt[1024]="%s %x %d ";
	DWORD	dwID;
	DWORD	dwSize;
	DWORD	dwData[250];

	DWORD	dwStackSize;
	DWORD	dwRet;
	DWORD	dwChar=0;

	dwRet=sscanf(cmdLine,"%s %x %c %d",chBuf,&dwID,&dwChar,&dwSize);
	if(dwRet==4 && (dwChar=='R'||dwChar=='r'))
	{
		if(dwSize>250)
		{
			printf("****** Size is too large (size<=250).\n");
			return;
		}

		srand(GetTickCount());

		//use random.
		for(DWORD i=0; i<dwSize; i++) dwData[i]=rand()%256;
	}
	else
	{
		//Input directly.
		dwRet=sscanf(cmdLine,chFmt,chBuf,&dwID,&dwSize);
		if(dwRet!=3)
		{
			OnHelp("? run");
			return;
		}
		
		if(dwSize>250)
		{
			printf("****** Size is too large (size<=250).\n");
			return;
		}
		
		for(DWORD i=0; i<dwSize; i++) strcat(chFmt,"%x ");
		
		dwStackSize=(dwSize+5)*4;
		
		_asm
		{
			mov		ecx,dwSize;
			lea		eax,dwData;
			mov		ebx,ecx;
			dec		ebx
			shl		ebx,2;
			add		eax,ebx;
			
loop0:
			push	eax;
			sub		eax,4
			loop	loop0;
			
			lea		eax,dwSize;
			push	eax;
			
			lea		eax,dwID;
			push	eax;
			
			lea		eax,chBuf;
			push	eax;
			
			lea		eax,chFmt;
			push	eax;
			
			mov		eax,cmdLine;
			push	eax;
			
			
			call	sscanf;
			mov		ebx,dwStackSize
			add		esp,ebx
			
			mov		dwRet,eax
		}
		
		while(dwRet!=dwSize+3)
		{
			dwRet+=scanf("%x",&dwData[dwRet-3]);			
		}
	}


	

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}		


	BYTE	byData[256];

	DICST_Before_Run_Data*	pBeforeRun=(DICST_Before_Run_Data*)byData;
	DICST_After_Run_Data*	pAfterRun=(DICST_After_Run_Data*)byData;
	
	pBeforeRun->RunID=(WORD)dwID;
	pBeforeRun->ParaSize=(WORD)dwSize;
	for(DWORD i=0; i<dwSize; i++) pBeforeRun->Para[i]=(BYTE)dwData[i];

	dwRet=g_DICNet5.Command(g_dwCurHandle,RUN,byData);

	if(dwRet)
	{
		printf("****** Run %04X failed, retCode=%08X(%u).\n",dwID,dwRet,dwRet);				
	}	

	else
	{
		printf("****** Return data size: %d\n",pAfterRun->ResultSize);

		DumpBuf(pAfterRun->Result,pAfterRun->ResultSize);

	}
}


//Dump Memory
void dump_memory(DWORD dwAddr,DWORD dwSize)
{	
	char 	buf[16];
	char	strbuf[20];
	BYTE	*pTemp,*pEnd;
	int		i;
	DWORD	dwHead;

	
	
	//calculate start and end address.	
	pTemp=(BYTE*)dwAddr;

	dwHead=(DWORD)pTemp;

	pEnd=pTemp+dwSize;
	strbuf[16]='\n';
	strbuf[17]=0;
	
	while(pTemp<pEnd)
	{
		//print address.
		sprintf(buf,"%08x  ",(DWORD)pTemp-dwHead);
		printf(buf);
		
		//printf binary.
		for(i=0; i<16; i++,pTemp++)
		{
			if((DWORD)pTemp<dwAddr||pTemp>=pEnd)
			{
				//not in range,show blank.
				printf("   "); strbuf[i]=' ';
			}
			else
			{
				//show memory.
				sprintf(buf,"%02X ",*pTemp);
				strbuf[i]=isprint(*pTemp)?*pTemp:'.';				
				if(i==7) buf[2]='-';
				printf(buf);
			}
		}
		
		//printf ASCII	
		printf("   ");	
		printf(strbuf);		
	}	
}



//Read file.
void DoRead(LPCTSTR cmdLine)
{
	char	chBuf[1024];
	char*	pData=NULL;
	DWORD	dwOffset,dwSize;
	int		iRead;
	DWORD	dwRet;

	//Get data form command line.
	iRead=sscanf(cmdLine,"%s %d %d",chBuf,&dwOffset,&dwSize);
	if(iRead!=3)
	{
		OnHelp("? read");
		return;
	}	

	if(dwOffset>0xFFFF||dwSize>0xFFFF)
	{
		printf("Offset<=65536,Size<=65535\n");
		return;
	}

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}
	
	pData=new char[dwSize+16];

	DICST_Before_Read_Data* pBRead=(DICST_Before_Read_Data*)pData;
	DICST_After_Read_Data*  pARead=(DICST_After_Read_Data*)pData;

	pBRead->offset=(WORD)dwOffset;
	pBRead->size=(WORD)dwSize;

	dwRet=g_DICNet5.Command(g_dwCurHandle,READ_FILE,pData);
	if(dwRet)
	{
		printf("****** Read failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		goto ENDFUNC;
	}

	dump_memory((DWORD)pARead->buffer,pARead->readedsize);
	



ENDFUNC:

	if(pData) delete[] pData;
	return;



}

//Read file.
void DoWrite(LPCTSTR cmdLine)
{
	char	chBuf[1024];
	char*	pData=NULL;
	DWORD	dwOffset,dwSize;
	int		iRead;
	DWORD	dwRet;

	//Get data form command line.
	iRead=sscanf(cmdLine,"%s %d %d",chBuf,&dwOffset,&dwSize);
	if(iRead!=3)
	{
		OnHelp("? write");
		return;
	}	

	if(dwOffset>0xFFFF||dwSize>0xFFFF)
	{
		printf("Offset<=65536,Size<=65535\n");
		return;
	}

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}
	
	pData=new char[dwSize+16];

	DICST_Write_Data* pWrite=(DICST_Write_Data*)pData;
	WORD* pBufData;

	pWrite->offset=(WORD)dwOffset;
	pWrite->size=(WORD)dwSize;
	pBufData=(WORD*)pWrite->buffer;
	for(DWORD i=0; i<=dwSize/2; i++,pBufData++) *pBufData=(WORD)i;	

	dwRet=g_DICNet5.Command(g_dwCurHandle,WRITE_FILE,pWrite);

	if(dwRet)
	{
		printf("****** Write failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		goto ENDFUNC;
	}	
	



ENDFUNC:

	if(pData) delete[] pData;
	return;



}


//Do Lock, Lock Device

void DoLock()
{
	DWORD	dwRet;
	BYTE	byBuf[256];



	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}

	//Lock Device
	dwRet=g_DICNet5.Command(g_dwCurHandle,NET_LOCK,byBuf);
	if(dwRet)
	{
		printf("****** Lock failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		goto ENDFUNC;
	}
	else
	{
		printf("****** Succeeded.\n");
	}

ENDFUNC:
	
	return;
}

void DoUnlock()
{
	DWORD	dwRet;
	BYTE	byBuf[256];

	//Check if selected handle.
	if(-1==g_dwCurHandle)
	{
		//No Selected.
		printf("****** No handle selected. Please use Open command to get a handle.\n");				
		return;
	}

	//Lock Device
	dwRet=g_DICNet5.Command(g_dwCurHandle,NET_UNLOCK,byBuf);
	if(dwRet)
	{
		printf("****** Unlock failed, retCode=%08X(%u).\n",dwRet,dwRet);				
		goto ENDFUNC;
	}
	else
	{
		printf("****** Succeeded.\n");
	}

ENDFUNC:
	
	return;
}



