// ==========================================================================
// RSA encryption/decryption
// Description: 
// If DES_RSA library is loaded, the DES, 3DES and RSA can be implemented by the following steps:
// (1) creat RSA key pair
// (2) encrypt/decrypt based on public key file ID or private key file ID
// Note:
// (1) the unit of key is bit, only support 1024 or 512 bits key
// (2) the default value of e is 65537, and cannot be changed
// (3) when encrypt data, if the data is longer than n, please set the high bit of data to 0
// ==========================================================================

#include "Rockey.h"
#include "stdio.h"
#include "iostream.h"
#include "dic32.h"

#define DIC_Command  ry.Command
#define DIC_Find     ry.Find
#define DIC_Open     ry.Open
#define DIC_Set      ry.Set
#define DIC_Get      ry.Get
#define DIC_Close    ry.Close


void main(int argc, char* argv[])
{
   CROCKEY ry;
   
 	int i, num, hic, errcode;
	char cmddata[256],dataSrc[128];
    
	char EncData[128]={0};
	WORD wRetLen;

	printf("Searching ROCKEY6 Smart ");
	num = DIC_Find();
	if (num <= 0)
	{
		printf("ROCKEY6 Smart not found\n");
		return;
	}
	printf("%d\n",num);

	printf("Open ROCKEY6 Smart\n");
	for (i=0;i<num;i++)
	{
		hic = DIC_Open(i, NULL);	// no specific reader name, NULL
		if (hic >= 0) break;		// find the first dongle
	}
	if (i == num)
	{
		printf("ROCKEY6 Smart not found\n");
		return;
	}

	memcpy(cmddata,"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",8);		
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("Verify Super Pass error : %8x\n",errcode);
	}
    DIC_Set(cmddata, VOLUME, BY_ARRAY, 0, "DEFAULT VOL");
	DIC_Set(cmddata, ATR, BY_ARRAY, 0, "DEFAULT ATR");
	errcode = DIC_Command(hic, FORMAT_CARD, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("Format card error: %8x\n",errcode);
	}

//////////////////////////////////////////////////////////////////////////
	// generate RSA key pair
	DIC_Set(cmddata, RSA_PUBKEYID, BY_VALUE, 0x7f01, NULL); //public key file ID
	DIC_Set(cmddata, RSA_PUBKEYSIZE, BY_VALUE, 1024, NULL); //public key size in bit
	DIC_Set(cmddata, RSA_PIVKEYID, BY_VALUE, 0x7f02, NULL); //private key file ID
	errcode = DIC_Command(hic, RSAGENKEY, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("RSA key error: %8x\n",errcode);
	}

	//plaintext
	for(i = 0; i<128;i++){
		dataSrc[i]=i;
	}

	//RSA encryption, either public key or private key
	DIC_Set(cmddata, RSA_KEYFILEID, BY_VALUE, 0x7f01, NULL); //use public key here
	DIC_Set(cmddata, RSA_DATA, BY_ARRAY | 128, 0, dataSrc);
	errcode = DIC_Command(hic, RSAENC, cmddata);
	if (errcode != SCARD_S_SUCCESS) 
	{
		printf("RSA enc error : %8x \n",errcode);
	}else
	{
		wRetLen = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, EncData);
		printf("Get %d Cipher text:\n",wRetLen);
		for(i= 0;i<wRetLen;i++){
			printf("%02x ",(BYTE)EncData[i]);
			if((i+1)%16 == 0) printf("\n");
		}
	}
	
	//RSA decryption
	DIC_Set(cmddata, RSA_KEYFILEID, BY_VALUE, 0x7f02, NULL);
	DIC_Set(cmddata, RSA_DATA, BY_ARRAY | 128, 0, EncData);
	errcode = DIC_Command(hic, RSADEC, cmddata);   
	if (errcode != SCARD_S_SUCCESS) 
	{
		printf("RSA dec error : %8x \n",errcode);
	}else
	{
		wRetLen = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, EncData);
		printf("Get %d plant text:\n",wRetLen);
		for(i= 0;i<wRetLen;i++){
			printf("%02x ",(BYTE)EncData[i]);
			if((i+1)%16 == 0) printf("\n");
		}
	}
		
	DIC_Close(hic);

    char a;
    cin>>a;

}
//---------------------------------------------------------------------------
 