! ==========================================================================
! Demo: The Super Password and Privilege Status
! Caution: no error check for the sake of briefness and understandability
! Tips:
! (1) ROCKEY5 developers will get a 8-bytes length super password with dongles.Super 
!     password verification is required for specific dongle operationsit may never 
!     be shared with end users
!     ROCEKY5 will be DICPR_SUPERPASS privilege status after Super password verification.
!	   Then developer can do operations below:
!
!     a) Create and delete files.
!     b) Create and delete internal files with API.
!     c) Format ROCKEY5 file system.
!     d) Reset the maximum retry numbers of super password verification.
!     e) Get remote upgrade password.

! (2) The super password of Demo ROCKEY5 is FF FF FF FF FF FF FF FF.
! (3) The GET_CARD_PRIVILEGE command will return the current privilege status of the dongle. 
!     Its return value is one byte in length, every bit of the byte represents different 
!     privilege status.
!     DICPR_NOPASS	0		! Password not verified status 
!     DICPR_SUPERPASS	4		! Super password verified status 
!     DICPR_REMOTEPASS	8		! Remote update password verified status

! (4) The SUPERPASS_STATUS macro of DIC_Get shows if the super password verification is 
!     successful. The return value of 1 represents successful verification, and 0 indicates 
!     super password verification failure. In same way,REMOTEPASS_STATUS macro detects whether the remote 
!     update password verification is successful.
! (5) You can set the maximum retry numbers when setting super password. Super password will be disabled if
!     if retry numbers exceed.
! ==========================================================================

!DEC$FREEFORM

INCLUDE 'head.F90'

integer*4 errcode 			! error code 
integer*4 num 				! number of founded card reader(s)
integer*4 hic 				! handle of Rockey5 device.
integer*4 i 
integer*4 ret_code 
CHARACTER*256 reader_name
CHARACTER cmddata(256) 	
CHARACTER buffer(256) 	
CHARACTER*20  p_null

	num = DIC_Find() 
	do i=0 ,num-1
		hic = DIC_Open(i, loc(reader_name)) 
		if (hic >= 0) then
		exit
		endif		! Stop searching once one is found
	enddo
	if (i == num)  then
        	stop
	endif

	! Get permission info
	errcode = DIC_Command(hic, GET_CARD_PRIVILEGE, loc(cmddata)) 
	ret_code = DIC_Get(loc(cmddata), SUPERPASS_STATUS, BY_VALUE, loc(p_null))
	if (ret_code/= 0) then
		write(*,*) 'Super password is verified'
	else 
	   write(*,*)'Super password is not verified'
	endif

	! Super password is 8 FFs
	do i =1,8
	cmddata(i) = char(255)
	enddo
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, loc(cmddata)) 

	! Get privilege information of the dongle
	errcode = DIC_Command(hic, GET_CARD_PRIVILEGE, loc(cmddata)) 
	ret_code= DIC_Get(loc(cmddata), SUPERPASS_STATUS, BY_VALUE, loc(p_null))
	if (ret_code/= 0) then
		write(*,*) 'Super password is verified'
	else 
	   write(*,*)'Super password is not verified'
	endif

	! Set new super passwrod
	! copy new password (0 1 2 3 4 5 6 7) to buffer
	! p3 of DIC_Set is maximum retry numbers(10)
	do i=1 ,8
	 buffer(i) = char(i+29)
	enddo 
	ret_code=DIC_Set(loc(cmddata), SUPERPASS_DATA, BY_ARRAY, 10, loc(buffer))
	errcode = DIC_Command(hic, SET_SUPER_PASS, loc(cmddata)) 

	! return to password not verified status
	errcode = DIC_Close(hic) 
	errcode = DIC_Open(hic, loc(p_null)) 

	! Check with new super passwrod.
	do i=1 ,8
	 cmddata(i) = char(i+29)
	enddo 
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, loc(cmddata)) 

	! Get privilege information of the dongle
	errcode = DIC_Command(hic, GET_CARD_PRIVILEGE, loc(cmddata)) 
	ret_code=DIC_Get(loc(cmddata), SUPERPASS_STATUS, BY_VALUE, loc(p_null))
	if (ret_code/= 0) then
		write(*,*) 'Super password is verified'
	else 
	   write(*,*)'Super password is not verified'
	endif

	! restore super password to 8 FFs.
	do i =1,8
	buffer(i) = char(255)
	enddo
	ret_code=DIC_Set(loc(cmddata), SUPERPASS_DATA, BY_ARRAY, 10, loc(buffer)) 
	errcode = DIC_Command(hic, SET_SUPER_PASS, loc(cmddata)) 

end

	