// ==========================================================================
// Demo: Secure File Transfer
// Caution: no error check for the sake of briefness and understandability
// Tips:
// The ROCKEY5 dongle supports Secure File Transfer (SFT). The Remote Update Management (RUM) 
// function will verify the user's identity, but it cannot update the dongle algorithm because the 
// end user does not have access to the super password which is required to write to the dongle. 
// The SFT process allows the software developer to securely update dongle algorithms or data files 
// without compromising the super password.
// See the SFT operational overview described below:
// 1. The software developer generates the "cryptography file" by using with own dongle.
// 2. Transfer the file to the user;
// 3. The user transfers the cryptography file to dongle with the program provided by the software 
//    developer. The dongle converts the file to clear text file internally.
// We adopts the following security policies:
// 1. The algorithms of cryptography files and clear text files are completely confidential, not 
//    open to anyone;
// 2. The generation of cryptography files and clear text files is 100% performed inside dongle, 
//    no other external work is involved;
// 3. The encryption / decryption key is related to dongle super password and the management code. 
//    It is not necessary to worry that some one else could create a cryptography file valid in the 
//    end user's dongle
// 4. The content of the cryptography file can not be modified, the file ID, its attribute, and 
//    others are part of the cryptography file, and they are all protected by the file check 
//    mechanism.
// Attention::
// 1. The software developer can only generate the cryptography file after super password 
//    verification. The verified super password is part of the encryption key. That is to say, 
//    the super password of the end user should be the same with the super password which the 
//    software developer uses to generate the cryptography file.
// 2. During the cryptography file generation process, a temporary file with the same name will be 
//    created in the software developer's dongle, because the IC card can transfer at most data of 
//    250 bytes once, and many files are larger than that size. Therefore, dongle should have enough 
//    space for the temporary file generation. The file will not be deleted automatically after the 
//    "cryptography file" generation call and the return of the content; you need to delete it 
//    manually.
// ==========================================================================
// Here two security files will be created: an executable file and a data file. The function 
// of the executable file is to read and return the content of the data file. 

//
// Executable file readdata.c, input parameters are the internal file IDs of dongle to be accessed
// Output parameters are the contents of the internal file of dongle.
//
//void main(void)
//{
// word fileID;
// word fileSize;
// byte b_fileSize;
// byte xdata out[30];
// byte xdata result[100],ret;

// get_input(&fileID,0,1,1);
// open_file(&fileID, 0);
// get_file_infor(out, 1);
// fileSize = (out[5]<<8)|out[4];
// read_file(0,result,fileSize);
// b_fileSize=(unsigned char)fileSize;

//set_response(b_fileSize, result);
// exit();
//}
// ==========================================================================

import java.io.*;

public class Sample13
{
	public Sample13()
	{
	}
	public static void main(String [] args) throws IOException
	{
		int i, num, hic=0, errcode;
		byte []cmddata=new byte[1024];	//Note: should be larger then DIST_SIZE+4 
		byte []buffer=new byte[1024];	//Note: should be large enough 
		int exesize, datasize, readsize;
		byte []c_exefile=new byte[1024], c_datafile=new byte[1024];
		int READDATA_SIZE= 276;
		int g_progReadData[]=
		{
		0x02,0x01,0x04,0x75,0x0c,0x00,0x75,0x0d,0x00,0x75,0x0e,0x08,0xe4,0xf5,0x0f,0x75,
		0x10,0x01,0x75,0x11,0x01,0x12,0x00,0xa0,0x75,0x0c,0x00,0x75,0x0d,0x00,0x75,0x0e,
		0x08,0xe4,0xf5,0x0f,0x12,0x00,0xbf,0x75,0x0c,0x01,0x75,0x0d,0x00,0x75,0x0e,0x00,
		0x75,0x0f,0x01,0x12,0x00,0xed,0x90,0x00,0x04,0xe0,0xfe,0xa3,0xe0,0xfd,0xee,0xf5,
		0x0a,0xed,0xf5,0x0b,0xe4,0xf5,0x0c,0xf5,0x0d,0x75,0x0e,0x01,0x75,0x0f,0x00,0x75,
		0x10,0x0c,0x85,0x0a,0x11,0x85,0x0b,0x12,0x12,0x00,0x7d,0xd3,0xe5,0x0b,0x94,0xff,
		0xe5,0x0a,0x94,0x00,0x40,0x04,0x7f,0xff,0x80,0x02,0xaf,0x0b,0x8f,0x0c,0x75,0x0d,
		0x01,0x75,0x0e,0x00,0x75,0x0f,0x0c,0x12,0x00,0xd6,0x02,0x01,0x10,0x12,0xff,0x00,
		0xe5,0x0c,0x24,0x00,0xe5,0x0d,0x24,0x01,0xe5,0x0e,0x24,0x02,0xe5,0x0f,0x24,0x03,
		0xe5,0x10,0x24,0x04,0xe5,0x11,0x24,0x05,0xe5,0x12,0x24,0x06,0x12,0xff,0x03,0x22,
		0x12,0xff,0x00,0xe5,0x0c,0x24,0x00,0xe5,0x0d,0x24,0x01,0xe5,0x0e,0x24,0x02,0xe5,
		0x0f,0x24,0x03,0xe5,0x10,0x24,0x04,0xe5,0x11,0x24,0x05,0x12,0xff,0x0b,0x22,0x12,
		0xff,0x00,0xe5,0x0c,0x24,0x00,0xe5,0x0d,0x24,0x01,0xe5,0x0e,0x24,0x02,0xe5,0x0f,
		0x24,0x03,0x12,0xff,0x02,0x22,0x12,0xff,0x00,0xe5,0x0d,0x24,0x00,0xe5,0x0e,0x24,
		0x01,0xe5,0x0f,0x24,0x02,0xe5,0x0c,0x24,0x03,0x12,0xff,0x0a,0x22,0x12,0xff,0x00,
		0xe5,0x0c,0x24,0x00,0xe5,0x0d,0x24,0x01,0xe5,0x0e,0x24,0x02,0xe5,0x0f,0x24,0x03,
		0x12,0xff,0x06,0x22,0x78,0x7f,0xe4,0xf6,0xd8,0xfd,0x75,0x81,0x12,0x02,0x00,0x03,
		0x12,0xff,0x0d,0x22,
		};
		
		JavaRockey Rockey=new JavaRockey();
		num = Rockey.DIC_Find();
		for (i=0;i<num;i++)
		{
			hic = Rockey.DIC_Open(i, null);
			if (hic >= 0) break;		//find the first ROCKEY
		}
		if (i == num) return;
	
		// a. Change to super user status (Only super users are allowed to create executable files)
		for (i=0;i<8;i++) cmddata[i] = (byte)0xff; // Super password is 8 FFs for demo ROCKEY
		errcode = Rockey.DIC_Command(hic, Rockey.CHECK_SUPER_PASS, cmddata);
	
    		// b. Format ROCKEY, all files and directorys will be cleared.
		//    You can set ATR and VOLUME here.
		Rockey.DIC_Set(cmddata, Rockey.VOLUME, Rockey.BY_ARRAY, 0, "DICSYSTEM\0".getBytes());
		Rockey.DIC_Set(cmddata, Rockey.ATR, Rockey.BY_ARRAY, 0, "DIC Co.\0".getBytes());
		errcode = Rockey.DIC_Command(hic, Rockey.FORMAT_CARD, cmddata);
	
		// c. Create the cryptography file of the executable file
		//    Difference between creating cryptography file and common file is that after setting the
		//    relevant file attributes FILE_DATA call is needed to transfer the file content.
		Rockey.DIC_Set(cmddata, Rockey.FILL, 512, 0, null);						// clear buffer
		Rockey.DIC_Set(cmddata, Rockey.FILE_ID, Rockey.BY_VALUE, 0x0023, null);				// file ID
		Rockey.DIC_Set(cmddata, Rockey.FILE_CLASS, Rockey.BY_VALUE, 0xff, null);			// file class
		Rockey.DIC_Set(cmddata, Rockey.FILE_ATTRIBUTE, Rockey.BY_VALUE, Rockey.FILEATTR_EXEC, null);	// file attr
		Rockey.DIC_Set(cmddata, Rockey.FILE_SIZE, Rockey.BY_VALUE, READDATA_SIZE, null);		// file size
		Rockey.DIC_Set(cmddata, Rockey.FILE_NAME, Rockey.BY_ARRAY, 0, "6088\0".getBytes());	
		for(i=0;i<READDATA_SIZE;i++) buffer[i]=(byte)g_progReadData[i];			
		Rockey.DIC_Set(cmddata, Rockey.FILE_DATA, Rockey.BY_ARRAY | READDATA_SIZE, 0, buffer);		// exe file name and content
		errcode = Rockey.DIC_Command(hic, Rockey.CRYPTOTEXT_FILE, cmddata);				
		exesize = Rockey.DIC_Get(cmddata, Rockey.FILE_DATA, Rockey.BY_ARRAY, c_exefile);
	
	     	// d.  Create the cryptography file of the data file, data content is 0-9 repeated 8 times.
		for (i=0;i<80;i++) buffer[i] = (byte)(0x30+(i%10));
		Rockey.DIC_Set(cmddata, Rockey.FILL, 256, 0, null);						// clear buffer
		Rockey.DIC_Set(cmddata, Rockey.FILE_ID, Rockey.BY_VALUE, 0x0057, null);				// file ID
		Rockey.DIC_Set(cmddata, Rockey.FILE_CLASS, Rockey.BY_VALUE, 0xff, null);			// file class
		Rockey.DIC_Set(cmddata, Rockey.FILE_ATTRIBUTE, Rockey.BY_VALUE, Rockey.FILEATTR_NORMAL, null);	// file attr
		Rockey.DIC_Set(cmddata, Rockey.FILE_SIZE, Rockey.BY_VALUE, 80, null);				// file size
		Rockey.DIC_Set(cmddata, Rockey.FILE_NAME, Rockey.BY_ARRAY, 0, "File1\0".getBytes());		// file name
		Rockey.DIC_Set(cmddata, Rockey.FILE_DATA, Rockey.BY_ARRAY | 80, 0, buffer);			// file content
		errcode = Rockey.DIC_Command(hic, Rockey.CRYPTOTEXT_FILE, cmddata);
		datasize = Rockey.DIC_Get(cmddata, Rockey.FILE_DATA, Rockey.BY_ARRAY, c_datafile);
	
		// e. Select the temporary file of the executable cryptography file($0023)
		Rockey.DIC_Set(cmddata, Rockey.FILE_ID, Rockey.BY_VALUE, 0x0023, null);
		errcode = Rockey.DIC_Command(hic, Rockey.SET_CURRENT_FILE, cmddata);
	
		// f. Delete temporary file of the executable cryptography file($0023)
		errcode = Rockey.DIC_Command(hic, Rockey.REMOVE_FILE, cmddata);
	
		// g. Select the temporary file of the data cryptography file($0057)
		Rockey.DIC_Set(cmddata, Rockey.FILE_ID, Rockey.BY_VALUE, 0x0057, null);
		errcode =Rockey.DIC_Command(hic, Rockey.SET_CURRENT_FILE, cmddata);
	
		// h. Delete the temporary file of the data cryptography file(0x0057)
		errcode = Rockey.DIC_Command(hic, Rockey.REMOVE_FILE, cmddata);
	
		// i. Lower the super password verified status to common user status, as it is unnecessary
		//    for super user to create the clear text file.
		errcode = Rockey.DIC_Close(hic);
		errcode = Rockey.DIC_Open(hic, null);
	
		// j. Create the clear text file of the exe-file
		for(i=0;i<exesize;i++) cmddata[i]=c_exefile[i];
		errcode = Rockey.DIC_Command(hic, Rockey.PLAINTEXT_FILE, cmddata);
	
		// k. Create the plaiintext file of the data file
		for(i=0;i<datasize;i++) cmddata[i]=c_datafile[i];
		errcode = Rockey.DIC_Command(hic, Rockey.PLAINTEXT_FILE, cmddata);
	
		// l. Run the executable file($0023), to get the content of the data file($0057)
		//    and return
		//	DIC_Set(buffer, 0, 2, 0x0057, null);					//Note , the file id should be inversed, because the card file system is different
		Rockey.DIC_Set(buffer, 0, 2, 0x0057, null);					// file ID
		Rockey.DIC_Set(cmddata, Rockey.RUN_DATA, 2, 0x6088, buffer);			// set data
		errcode = Rockey.DIC_Command(hic, Rockey.RUN, cmddata);
		readsize = Rockey.DIC_Get(cmddata, Rockey.RUN_DATA, Rockey.BY_ARRAY, buffer);	// get data
		for (i=0;i<readsize;i++)
		{
			System.out.print((char)buffer[i]);
			if (((i+1)%10) == 0) System.out.println();
		}
	
		// m. close
		errcode = Rockey.DIC_Close(hic);
	}

}