// ==========================================================================
// RSA encryption/decryption
// Description: 
// If DES_RSA library is loaded, the DES, 3DES and RSA can be implemented by the following steps:
// (1) creat RSA key pair
// (2) encrypt/decrypt based on public key file ID or private key file ID
// Note:
// (1) the unit of key is bit, only support 1024 or 512 bits key
// (2) the default value of e is 65537, and cannot be changed
// (3) when encrypt data, if the data is longer than n, please set the high bit of data to 0
// ==========================================================================


#include <windows.h>
#include <stdio.h>
#include "dic32.h"

void main(int argc, char* argv[])
{
	int i, num, hic, errcode;
	char cmddata[256],dataSrc[128];
    
	char EncData[128]={0};
	WORD wRetLen;

	printf("searching ROCKEY: ");
	num = DIC_Find();
	if (num <= 0)
	{
		printf("No ROCKEY found\n");
		return;
	}
	printf("%d\n",num);

	printf("Open ROCKEY\n");
	for (i=0;i<num;i++)
	{
		hic = DIC_Open(i, NULL);	
		if (hic >= 0) break;		// find the first ROCKEY
	}
	if (i == num)
	{
		printf("No ROCKEY found\n");
		return;
	}

	memcpy(cmddata,"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",8);		
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("Verify Super Pass error : %8x\n",errcode);
	}
    

	//////////////////////////////////////////////////////////////////////////
	//generate RSA key pair
	DIC_Set(cmddata, RSA_PUBKEYID, BY_VALUE, 0x7f01, NULL); //public key file ID
	DIC_Set(cmddata, RSA_PUBKEYSIZE, BY_VALUE, 1024, NULL); //public key size in bit
	DIC_Set(cmddata, RSA_PIVKEYID, BY_VALUE, 0x7f02, NULL); //private key file ID
	errcode = DIC_Command(hic, RSAGENKEY, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("RSA key error: %8x\n",errcode);
	}

	//set plaintext
	for(i = 0; i<128;i++){
		dataSrc[i]=i;
	}
	dataSrc[124]= 0; //the high byte is 0
	
	//RSA encryption, either public key or private key
	DIC_Set(cmddata, RSA_KEYFILEID, BY_VALUE, 0x7f01, NULL); //use public key here
	DIC_Set(cmddata, RSA_DATA, BY_ARRAY | 128, 0, dataSrc);
	errcode = DIC_Command(hic, RSAENC, cmddata);
	if (errcode != SCARD_S_SUCCESS) 
	{
		printf("RSA enc error : %8x \n",errcode);
	}else
	{
		wRetLen = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, EncData);
		printf("Get %d Cipher text:\n",wRetLen);
		for(i= 0;i<wRetLen;i++){
			printf("%02x ",(BYTE)EncData[i]);
			if((i+1)%16 == 0) printf("\n");
		}
	}
	
	//RSA decrypt
	DIC_Set(cmddata, RSA_KEYFILEID, BY_VALUE, 0x7f02, NULL);
	DIC_Set(cmddata, RSA_DATA, BY_ARRAY | 128, 0, EncData);
	errcode = DIC_Command(hic, RSADEC, cmddata);   
	if (errcode != SCARD_S_SUCCESS) 
	{
		printf("RSA dec error : %8x \n",errcode);
	}else
	{
		wRetLen = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, EncData);
		printf("Get %d plant text:\n",wRetLen);
		for(i= 0;i<wRetLen;i++){
			printf("%02x ",(BYTE)EncData[i]);
			if((i+1)%16 == 0) printf("\n");
		}
	}
		
	DIC_Close(hic);
}

