using System;
using System.Runtime.InteropServices;
using FeiTian;

namespace CSTest
{
	/// <summary>
	/// ROCKEY SMART sample in C#
	/// please include ROCKEY com control
	/// </summary>	
	class CSTest
	{

		enum Ry6sFileAttr:byte
		{
			NORMAL=0x00,
			EXEC=0x10,
			DIR=0x20,
			UPIGNORE=0x40,
			INTERNAL=0x80
		};



		/// <summary>
		/// Main entry point.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			//Local variants.
			CRockeySmart 	Rs=new CRockeySmart();
			uint		uiMaxDevs,uiDevID;
			string		csDevName;

			try
			{
				//Demo1:Find. 
				//Get maximum device numbers.
				uiMaxDevs=Rs.Find();
				Console.WriteLine("ROCKEY6 SMART Find: found {0} devices.",uiMaxDevs);

				//Demo2:Open
				//Open device specified by user.
				Console.WriteLine("Open which device? (input number,>=0,<{0}",uiMaxDevs);				
				uiDevID=Convert.ToUInt32(Console.ReadLine());				
				Console.WriteLine("******** Opening device {0}...",uiDevID);
				csDevName=Rs.Open(uiDevID);

				Console.WriteLine("Device {0} opened, name is {1}\n",uiDevID,csDevName);

				//You can use OpenByName is you know card name.
				/*
				 *	Such as:
				 *  uing	uiDevID;
				 *	string	csCardname="FT SCR2000 0";
				 *  
				 *  uiDevID=Ry5.OpenByName(csCardname);	
				 * 
				 *  ......
				 */

				

				//Demo3:Check super password.
				Console.WriteLine("******** Check super password FF FF FF FF FF FF FF FF");
				byte[] bySuperPassword=new byte[8]{0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
				try
				{
					Rs.CheckSuperPassword(uiDevID,bySuperPassword);
				}
				catch(COMException e)
				{
					Console.WriteLine("super password is not FF FF FF FF FF FF FF FF,please modify it");

					return;					
				}
				Console.WriteLine("[OK]\n");

				//Demo4: Modify super password.
				byte[] byNewSuperPassword=new Byte[8]{0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef};

				Console.WriteLine("******** modify super password  to 01 23 45 67 89 AB CD EF");
				Rs.SetSuperPassword(uiDevID,15,byNewSuperPassword);
				Console.WriteLine("[OK]\n");

				//Set back old password.
				Console.WriteLine("******** modify super password  to FF FF FF FF FF FF FF FF");
				Rs.SetSuperPassword(uiDevID,15,bySuperPassword);
				Console.WriteLine("[OK]\n");
				



				//Demo5:FormatCard.
				Console.WriteLine("******** Format card, Volume:Volumn_Test, ATR:ATR_Test");
				Rs.FormatCard(uiDevID,"Volumn_Test","ATR_Test");
				Console.WriteLine("[OK]\n");

				//Demo6: Get Card info
				string csVolumn,csATR;

				Console.WriteLine("******** Get card information");

				Rs.GetCardInfo(uiDevID,out csVolumn,out csATR);

				Console.WriteLine("Volumn: {0}, ATR: {1}",csVolumn,csATR);

				Console.WriteLine("[OK]\n");

				//Demo7: Get Hardware info.
				uint uiFactoryTime,uiHardSerial,uiShipTime,uiCOSVer;

				Console.WriteLine("******** Get hardware information");

				Rs.GetHardwareInfo(uiDevID,out uiFactoryTime,out uiHardSerial,out uiShipTime,out uiCOSVer);
				Console.WriteLine("FactoryTime:{0:X8}",uiFactoryTime);
				Console.WriteLine("HardSerial :{0:X8}",uiHardSerial);
				Console.WriteLine("ShipTime   :{0:X8}",uiShipTime);
				Console.WriteLine("COS Version:{0:X8}",uiCOSVer);

				Console.WriteLine("[OK]\n");

				//Demo8: Get Manager code.
				ushort Zone,Agent,User1,User2;

				Console.WriteLine("******** Get Manager code");

				Rs.GetManagerCode(uiDevID,out Zone,out Agent,out User1,out User2);

				Console.WriteLine("Manager code: {0:X4},{1:X4},{2:X4},{3:X4}",Zone,Agent,User1,User2);

				Console.WriteLine("[OK]\n");

				//Demo9: Get Card privilege.
				byte byPrivilege;

				Console.WriteLine("******** Get card privilege");
				byPrivilege=Rs.GetCardPrivilege(uiDevID);
				Console.WriteLine("Current card privilege is {0:X2}",byPrivilege);
				Console.WriteLine("[OK]\n");

				//Demo10: Set Remote password. ONLY CAN be used after card format.				

				byte[] byInitRemotePW=new byte[8];
				
				for(int i=0; i<8; i++) byInitRemotePW[i]=(byte)i;

				Console.WriteLine("******** Set Remote information.(Tag=0x1001,PW= 00 01 02 03 04 05 06 07)");
				Rs.SetRemoteInfo(uiDevID,0x1001,byInitRemotePW);
				Console.WriteLine("[OK]\n");

				//Demo11:Get Romote information.
				uint	uiRemoteTag;
				byte[]	byPassword;
				object	obPW;

				Console.WriteLine("******** Get remote information");
				Rs.GetRemoteInfo(uiDevID,out uiRemoteTag,out obPW);
				byPassword=(byte[])obPW;

				Console.Write("RemoteTag is {0:X8}, Remote password is ",uiRemoteTag);
				foreach(byte i in byPassword)
				{
					Console.Write("{0:X2} ",i);
				}
				Console.WriteLine(".");
				Console.WriteLine("[OK]\n");

				//Demo12: Get upgrade remote password.
				byte[] byNewRemotePW;

				Console.WriteLine("******** Get upgrade remote password");
				byNewRemotePW=(byte[])Rs.GetUpgradeRemotePassword(uiDevID,0x1002,uiHardSerial,byPassword);

				Console.Write("New remote password is: ");
				foreach(byte i in byNewRemotePW)
				{
					Console.Write("{0:X2} ",i);
				}
				Console.WriteLine(".");
				Console.WriteLine("[OK]\n");

				//Demo13: Check Upgrade password.
				Console.WriteLine("******** Check upgrade remote information");
				Rs.CheckRemoteInfo(uiDevID,0x1002,byNewRemotePW);
				Console.WriteLine("[OK]\n");

				//Check current remote information.

				Console.WriteLine("******** Get current remote information");
				Rs.GetRemoteInfo(uiDevID,out uiRemoteTag,out obPW);
				byPassword=(byte[])obPW;

				Console.Write("Currrent RemoteTag is {0:X8}, Remote password is ",uiRemoteTag);
				foreach(byte i in byPassword)
				{
					Console.Write("{0:X2} ",i);
				}
				Console.WriteLine(".");			

				byPrivilege=Rs.GetCardPrivilege(uiDevID);
				Console.WriteLine("[OK], privilege change to {0:X2}\n",byPrivilege);


				//Demo14:Create directory.
				Console.WriteLine("******** Create a new Directory,ID:0x3001,Name:TestDir1");
				Rs.CreateDir(uiDevID,0x3001,0,0,"TestDir1");
				Console.WriteLine("[OK]\n");				


				//Demo15:Set current dir to 0x3001
				Console.WriteLine("******** cd 3001");
				Rs.SetCurrentDir(uiDevID,0x3001);
				Console.WriteLine("[OK]\n");	
				

				//Demo16:Get Current Dir
				ushort	usDirID;
				byte	byDirClass,byDirPrivilege;
				string	csDirName;

				Console.WriteLine("******** Get current directory information");
				Rs.GetCurrentDir(uiDevID,out usDirID,out byDirClass,out byDirPrivilege,out csDirName);

				Console.WriteLine("Current Dir information(name,ID,class,atr&privilege):");
				Console.WriteLine("{0,-16} {1:X4} {2:X2} {3:X2}",csDirName,usDirID,byDirClass,byDirPrivilege);
				Console.WriteLine("[OK]\n");

				//Demo17:Get Parent Dir.
				Console.WriteLine("******** Get parent directory information");
				Rs.GetParentDir(uiDevID,out usDirID,out byDirClass,out byDirPrivilege,out csDirName);

				Console.WriteLine("Parent Dir information(name,ID,class,atr&privilege):");
				Console.WriteLine("{0,-16} {1:X4} {2:X2} {3:X2}",csDirName,usDirID,byDirClass,byDirPrivilege);
				Console.WriteLine("[OK]\n");

				//Demo18:Set parent Dir.
				Console.WriteLine("******** Set to parent directory");
				Rs.SetParentDir(uiDevID);
				Console.WriteLine("[OK]\n");
	
				//Demo19:Create a new file.
				Console.WriteLine("******** Create a new file,ID:0x1001,Name:TestFile1,Size:512");
				Rs.CreateFileA(uiDevID,0x1001,0,0,512,"TestFile1");
				Console.WriteLine("[OK]\n");

				Console.WriteLine("******** Create a new file,ID:0x1002,Name:TestFile2,Size:256");
				Rs.CreateFileA(uiDevID,0x1002,0,0,256,"TestFile2");
				Console.WriteLine("[OK]\n");

				//Demo20:SetCurrentFile
				Console.WriteLine("******** Set current file to 0x1001");
				Rs.SetCurrentFile(uiDevID,0x1001);
				Console.WriteLine("[OK]\n");

				//Demo21:GetCurrentFile				
				ushort	usFileID,usFileSize;
				byte	byFileClass,byFilePrivilege;
				string	csFileName;

				Console.WriteLine("******** Get current file information(name,ID,class,size,atr&privilege):");

				Rs.GetCurrentFile(uiDevID,out usFileID,out byFileClass,out byFilePrivilege,out usFileSize,out csFileName);
				
				Console.WriteLine("{0,-16} {1:X4} {2:X2} {3:X4} {4:X2}",csFileName,usFileID,byFileClass,usFileSize,byFilePrivilege);
				Console.WriteLine("[OK]\n");

				//Demo22:Write file.
				byte[] byFileData=new Byte[512];

				for(int i=0; i<256; i++)
				{
					byFileData[i*2]=(byte)(i);
					byFileData[i*2+1]=(byte)(i/256);
				}

				Console.WriteLine("******** Write file,512 bytes");
				Rs.WriteFile(uiDevID,0,512,byFileData);
				Console.WriteLine("[OK]\n");


				//Demo23:ListDir.
				Console.WriteLine("******** List files in current directory.(name,ID,class,size,atr&privilege)");
				
				byte iIndex=0;
				try
				{				

					while(true)
					{
						Rs.ListDir(uiDevID,iIndex,out usFileID,out byFileClass,out byFilePrivilege,out usFileSize,out csFileName);

						Console.WriteLine("{0,-16} {1:X4} {2:X2} {3:X4} {4:X2}",csFileName,usFileID,byFileClass,usFileSize,byFilePrivilege);				

						iIndex++;
					}				

				}
				catch(COMException e)
				{				
					Console.WriteLine("Totle: {0} files\n",iIndex);
				}			

				//Demo24,Read file.
				ushort	usReadSize;
				object	obFileData;
				byte[]	byReadData;

				Console.WriteLine("******** Read file,512 bytes");
				usReadSize=Rs.ReadFile(uiDevID,0,512,out obFileData);
				byReadData=(byte[])obFileData;
				
				for(ushort i=0; i<usReadSize; i++)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byReadData[i]);
				}				

				Console.WriteLine("\n[OK], Totle read:{0}\n",usReadSize);

				//Demo25,Remove File.
				Console.WriteLine("******** Delete file 0x1001");
				Rs.RemoveFile(uiDevID);
				Console.WriteLine("[OK]\n");

				Console.WriteLine("******** Delete file 0x1002");
				Rs.SetCurrentFile(uiDevID,0x1002);
				Rs.RemoveFile(uiDevID);
				Console.WriteLine("[OK]\n");

				//Demo26,Remove Dir.
				Console.WriteLine("******** Delete dir 0x3001");
				Rs.SetCurrentDir(uiDevID,0x3001);
				Rs.RemoveDir(uiDevID);
				Console.WriteLine("[OK]\n");  
				

				//Demo27,Get a eight-byte random number.				
				
				byte[] byarrRandom;

				Console.WriteLine("******** Get 8-byte Random number.");
				byarrRandom=(byte[])Rs.Random(uiDevID,8);

				Console.Write("Random is: ");
				foreach(byte i in byarrRandom)
				{
					Console.Write("{0:X2} ",i);
				}
				Console.WriteLine("\n[OK]\n");

				//Demo28, Crypto an executable file. this file can calculate n!.
				const ushort usExeFileSize=252;
				byte[] byExeFileData=new Byte[usExeFileSize]
				{
					0x02,0x00,0xEC,0x75,0x0D,0x00,0x75,0x0E,0x00,0x75,0x0F,0x08,0xE4,0xF5,0x10,0xF5
          ,0x11,0x75,0x12,0x01,0x12,0x00,0xB6,0xE5,0x08,0x60,0x0A,0xE4,0x75,0x0C,0x01,0xF5
          ,0x0B,0xF5,0x0A,0xF5,0x09,0x90,0x00,0x01,0xD3,0xE5,0x82,0x95,0x08,0x74,0x80,0xF8
          ,0x65,0x83,0x98,0x50,0x20,0xAF,0x82,0xAE,0x83,0xEE,0x33,0x95,0xE0,0xFD,0xFC,0xAB
                    ,0x0C,0xAA,0x0B,0xA9,0x0A,0xA8,0x09,0x12,0x00,0x67,0x8F,0x0C,0x8E,0x0B,0x8D,0x0A
                    ,0x8C,0x09,0xA3,0x80,0xD3,0x75,0x0D,0x04,0x75,0x0E,0x00,0x75,0x0F,0x00,0x75,0x10
                    ,0x09,0x12,0x00,0xD5,0x02,0x00,0xF8,0xE8,0x8F,0xF0,0xA4,0xCC,0x8B,0xF0,0xA4,0x2C
                    ,0xFC,0xE9,0x8E,0xF0,0xA4,0x2C,0xFC,0x8A,0xF0,0xED,0xA4,0x2C,0xFC,0xEA,0x8E,0xF0
                    ,0xA4,0xCD,0xA8,0xF0,0x8B,0xF0,0xA4,0x2D,0xCC,0x38,0x25,0xF0,0xFD,0xE9,0x8F,0xF0
                    ,0xA4,0x2C,0xCD,0x35,0xF0,0xFC,0xEB,0x8E,0xF0,0xA4,0xFE,0xA9,0xF0,0xEB,0x8F,0xF0
                    ,0xA4,0xCF,0xC5,0xF0,0x2E,0xCD,0x39,0xFE,0xE4,0x3C,0xFC,0xEA,0xA4,0x2D,0xCE,0x35
                    ,0xF0,0xFD,0xE4,0x3C,0xFC,0x22,0x12,0xFF,0x00,0xE5,0x0D,0x24,0x00,0xE5,0x0E,0x24
                    ,0x01,0xE5,0x0F,0x24,0x02,0xE5,0x10,0x24,0x03,0xE5,0x11,0x24,0x04,0xE5,0x12,0x24
                    ,0x05,0x12,0xFF,0x0B,0x22,0x12,0xFF,0x00,0xE5,0x0E,0x24,0x00,0xE5,0x0F,0x24,0x01
                    ,0xE5,0x10,0x24,0x02,0xE5,0x0D,0x24,0x03,0x12,0xFF,0x0A,0x22,0x78,0x7F,0xE4,0xF6
                    ,0xD8,0xFD,0x75,0x81,0x12,0x02,0x00,0x03,0x12,0xFF,0x0D,0x22
				};

				ushort usCryptDataLen;
				object obExeDataAfterCrypt;

				Console.WriteLine("******** Crypto an exe file. length:{0}.",usExeFileSize);
				usCryptDataLen=Rs.CryptoTextFile(uiDevID,0x2008,0xFF,(byte)Ry6sFileAttr.EXEC,usExeFileSize,byExeFileData,"2008",
					out obExeDataAfterCrypt);
				Console.WriteLine("[OK],return length is {0}\n",usCryptDataLen);

				//ok,file created. we delete it.
				Rs.SetCurrentFile(uiDevID,0x2008);
				Rs.RemoveFile(uiDevID);

				//Demo29, PlainTextFile. that will create a new file in card.
				Console.WriteLine("******** Plain crypted file to card. Name:2008");

				Rs.PlainTextFile(uiDevID,obExeDataAfterCrypt);

				Console.WriteLine("[OK]");


				//Demo30, Close and reopen to return normal privilege.after that ,you can run exe-file.			
				Console.WriteLine("******** Return to normal privilege");
				Rs.Close(uiDevID);
				Rs.Open(uiDevID);
				Console.WriteLine("[OK]");

				//Demo31,Run program, RunID=Name=0x2008;
				//Calculate n!. input 4 bytes, and output 4bytes.
				
				byte[] byInputData=new byte[1]{0x03};
				object obRunRet;
				byte[] byRunRet;
				ushort usRunRetSize;

				Console.WriteLine("******** Run to calcute 3!=6");
				usRunRetSize=Rs.Run(uiDevID,0x2008,1,byInputData,out obRunRet);

				byRunRet=(byte[])obRunRet;

				Console.Write("Return data: ");
				foreach(byte i in byRunRet)
				{
					Console.Write("{0:X2} ",i);
				}
				Console.WriteLine("\n[OK],return size is {0}\n",usRunRetSize);			

				Console.WriteLine("Create Rsa Key Public Key 1001,Private Key 1002,Key length 1024 \n");

				Rs.CheckSuperPassword(uiDevID,bySuperPassword);

				Rs.RsaGenKey(uiDevID,0x1001,1024,0x1002);
            	
				Console.WriteLine("Begin Rsa Encrypt :\n");
				object obData;
				byte []byData=new byte[128];

				for(byte i=0;i<128;i++)
				{
					byData[i]=i;
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}
				obData=(object)byData;
				Rs.RsaEnc(uiDevID,0x1001,128,ref obData);

				Console.WriteLine("\nRsa Encrypt End:\n");
				byData=(byte[])obData;

				for(byte i=0;i<128;i++)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}
				Console.WriteLine("\n Data After Rsa Decrypt \n");
				Rs.RsaDec(uiDevID,0x1002,128,ref obData);

				byData=(byte[])obData;

				for(byte i=0;i<128;i++)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}

				Rs.CreateFileA(uiDevID,0x1003,0xff,0x80,28,"desKey");
				byte[] writeData={1,8,1,2,3,4,5,6,7,8,2,16,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16};
				Rs.WriteFile(uiDevID,0,28,writeData);

				
				Console.WriteLine("\n Data After Des Encrypt \n");
				Rs.DesEnc(uiDevID,128,0x1003,1,ref obData);
				byData=(byte[])obData;
				for(byte i=0;i<128;i++)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}
				Console.WriteLine("\n Data After Des Decrypt \n");
				Rs.DesDec(uiDevID,128,0x1003,1,ref obData);

				byData=(byte[])obData;
				foreach(byte i in byData)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}

				Console.WriteLine("\n Data After tDes Encrypt \n");
				Rs.DesEnc(uiDevID,128,0x1003,2,ref obData);
				byData=(byte[])obData;
				for(byte i=0;i<128;i++)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}
				Console.WriteLine("\n Data After tDes Decrypt \n");
				Rs.DesDec(uiDevID,128,0x1003,2,ref obData);

				byData=(byte[])obData;
				foreach(byte i in byData)
				{
					if(i%16==0) 
					{
						Console.WriteLine("");
						Console.Write("{0:X4}  ",i);
					}

					Console.Write("{0:X2} ",byData[i]);
				}
				
				Console.WriteLine("\n Set Counter 5\n");
				Rs.SetCounter(uiDevID,5);
				{
					int i=0;
					try
					{
						for(i=0;i<100;i++)
						{
							Rs.StepCounter(uiDevID);
						}
					}
					catch(COMException e)
					{
						Console.WriteLine("The Counter is {0:X4}\n",i);
					}
				}
				Rs.FormatCard(uiDevID,"DEFAULT VOLUME","DEFAULT ATR");
				//Close device.
				Rs.Close(uiDevID);

							
				

			}
			catch(COMException e)
			{				
				Console.WriteLine("Failed.{0}",e);
			}			

			
		}		
	}
}
